/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/data/TimeZones'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/cmd/ShowCommand'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , FormPanel
          , TimeZones
          , Command
          , ButtonFactory
          , ShowCommand
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'ClockHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.ClockHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.ClockHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ClockHandler.prototype ).constructor = tsa.cmd.handlers.ClockHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.ClockHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n               = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea        = new ContentArea ( { title   : i18n.labelClockHeading
                                                                                          , helpDoc : 'sshelp_clock_panel.html' } );
      /** @type {tsa.widgets.FormPanel}     */ var formPanel          = new FormPanel( { title  : i18n.labelClockTzHeading
                                                                                       , action : 'clock/ajaxUpdate' } );
      /** @type {tsa.widgets.FormPanel}     */ var formPanelTOption   = new FormPanel( { title  : i18n.labelClockTimeAssignHeading } );
      /** @type {tsa.widgets.FormPanel}     */ var formPanelDTime     = new FormPanel( { title  : i18n.labelClockDtHeading } );
      /** @type {tsa.widgets.FormPanel}     */ var formPanelNTP       = new FormPanel( { title  : i18n.labelClockNtpHeading} );
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer    = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getClockCommand    = null;
      /** @type {Element}                   */ var selectGTM          = document.createElement( 'select' );
      /** @type {Element}                   */ var selectDST          = document.createElement( 'select' );
      /** @type {Element}                   */ var selectSLC          = document.createElement( 'select' );
      
      /* Appending the form to the Content Area. Notice the forms will be linked
       * ( formPanel -> formPanelToption -> formPanelDoption -> formPanelNTP )
       * so only the first (head) form will be appended.
       */ 
      contentArea.append( formPanel.getDomNode() );
      
      formPanel        .linkTo( formPanelTOption );
      formPanelTOption .linkTo( formPanelDTime   );
      formPanelDTime   .linkTo( formPanelNTP     );
      
      // Forms descriptions...
      formPanel        .setDescription( i18n.labelClockTzDesc         );
      formPanelTOption .setDescription( i18n.labelClockTimeAssignDesc );
      formPanelDTime   .setDescription( i18n.labelClockDtDesc         );
      formPanelNTP     .setDescription( i18n.labelClockNtpDesc        );
      
      // Auxiliary function to disable inputs depending on NTP settings...
      /** @type {function(string)} */ var disableInputs = function( assignment )
      {
        // "ntp" to disable ntp fields, manual fields will be disabled otherwise.
        
        /** @type {boolean} */ var isNTP = ( assignment === 'ntp' );
        
        $( '[name="server1"]' ).prop('disabled' , !isNTP );
        $( '[name="server2"]' ).prop('disabled' , !isNTP );
        $( '[name="date"]'    ).prop('disabled' , isNTP  );
        $( '[name="time"]'    ).prop('disabled' , isNTP  );
      };
      
      // Timezone selector properties...
      selectGTM.name = "timezone";
      selectGTM.id   = "timezone";
      
      // Populating the timezone selector and appending it to the form...
      $.each( TimeZones.getTimeZones( 'en' ), function( timeZoneIdx, timeZone )
      {
        /** @type {Element} */ var opt = document.createElement( 'option' );
        opt.value = timeZone.id;
        $( opt ).text( timeZone.name.en ); 
        $( selectGTM ).append( opt );
      } );
     
      formPanel.appendFieldRow( { label    : i18n.labelClockGmtOffset
                                , required : true
                                , element  : selectGTM
                                } );
      
      // DST selector properties...
      selectDST.name = "dst";
      selectDST.id   = "dst";
      
      // Populating the DST selector and appending it to the form...
      $.each( [ { value: 'auto'   , text: i18n.labelClockDstAdjustAuto   }
              , { value: 'manual' , text: i18n.labelClockDstAdjustManual } ]
            , function( optionDataIndex, optionData )
              {
                /** @type {Element} */ var opt = document.createElement( 'option' );
                
                opt.value = optionData.value;
                $( opt ).text( optionData.text );
                $( selectDST ).append( opt );
              } );
      
      formPanel.appendFieldRow( { label    : i18n.labelClockDstAdjust
                                , required : true
                                , element  : selectDST
                                } );
      
      // SLC selector properties...
      selectSLC.name = "assignment";
      selectSLC.id   = "assignment";
      
      // Populating the SLC selector and appending it to the form...
      $.each( [ { value: 'manual' , text: i18n.labelClockTimeAssignManual }
              , { value: 'ntp'    , text: i18n.labelClockTimeAssignNtp    } ]
            , function( optionDataIndex, optionData )
              {
                /** @type {Element} */ var opt = document.createElement( 'option' );
                
                opt.value = optionData.value;
                $( opt ).text( optionData.text );
                $( selectSLC ).append( opt );
              } );
      
      $( selectSLC ).on( 'change', function()
      {
        disableInputs( this.value );
      } );
      
      formPanelTOption.appendFieldRow( { label    : i18n.labelClockTimeAssign
                                       , required : true
                                       , element  : selectSLC
                                       } );
      
      // Appending the date and time text fields...
      formPanelDTime.appendFieldRow( { label    : i18n.labelClockDtDate
                                     , required : true
                                     , hint     : i18n.labelClockDtDateDesc
                                     , element  : $( '<input type="text" size="50" name="date" id="date" value="" required/>' ).get(0)
                                     } );
      formPanelDTime.appendFieldRow( { label    : i18n.labelClockDtTime
                                     , required : true
                                     , hint     : i18n.labelClockDtTimeDesc
                                     , element  : $( '<input type="text" size="50" name="time" id="time" value="" required/>' ).get(0)
                                     } );
      
      // Appending the NTP servers text fields...
      formPanelNTP.appendFieldRow( { label    : i18n.labelClockNtpServer1
                                   , required : true
                                   , hint     : i18n.labelClockNtpServer1Desc
                                   , element  : $( '<input type="text" size="50" name="server1" id="server1" value="" required/>' ).get(0)
                                   } );
      formPanelNTP.appendFieldRow( { label    : i18n.labelClockNtpServer2
                                   , required : false
                                   , hint     : i18n.labelClockNtpServer2Desc
                                   , element  : $( '<input type="text" size="50" name="server2" id="server2" value="" required/>' ).get(0)
                                   } );
      
      // Building the buttons section and appending it to the content area...
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonSave
          , onclick : function( event )
                      {
                        $( formPanel ).on( 'executionSucceeded', function( e )
                        {
                         //if reboot needed ask user if restart now
                          if(e.response.payload["reboot"])
                          {
                            if ( confirm('System restart is required. Restart system now?') ) {
                              tsa.cmd.handlers.ClockHandler.rebootSystem('auto');
                            } else {
                              tsa.cmd.handlers.ClockHandler.rebootSystem('user');
                            }
                          }
                          ( new ShowCommand( { action : 'main/index' , clearSystemMessages : false  } ) ).execute();
                        } );
                        
                        formPanel.submit();
                      }
          }
        )
      );
      
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );
      
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'cancel'
          , grayout : true
          , label   : i18n.buttonCancel
          , onclick : function( event )
                      {
                        ( new ShowCommand( { action : 'main/index' , clearSystemMessages : false  } ) ).execute();
                      }
          }
        )
      );
      
      contentArea.append( buttonContainer );
      
      // Time to show the content area...
      this.show( contentArea.getDomNode() );
      
      // Initializing the command that gets the current clock data from the server...
      getClockCommand = new Command( { action              : 'clock/getClock'
                                     , clearSystemMessages : false } );
      
      // ...defining how to handle the server clock data...
      $( getClockCommand ).on( 'executionSucceeded', function( e )
      {
        for( /** @type {string} */ var key in e.response.payload ) 
        {
          if( key === 'oldAssignment' || key === 'oldDate' || key === 'oldTime' || key === 'restart' )
          {
            formPanel.putHiddenInput( key, e.response.payload[ key ] );
          } else {
            $( '[name="' + key + '"]' ).val( e.response.payload[ key ] ); 
          }
        }
        
        disableInputs( e.response.payload[ 'assignment' ] );
      } );
      
      // ...defining how to handle server errors...
      $( formPanel ).on( 'executionFailed', function( event ){
        if( TV.typeIsObject( typeof event.response.payload.errors ) )
        {
          for( /** @type {string} */ var fieldName in event.response.payload.errors )
          {
            formPanel.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
          }
          delete event.response.payload.errors;
        }
      } );
      
      // Executing the command to retrieve clock data from the server...
      getClockCommand.execute(); 
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Request for restarting system.
     * @param {string} restart Restart mode. Use either:
     *                         'auto' : Restart will be initiated by the system.
     *                         'user' : Restart should be initiated by the user.
     */
    tsa.cmd.handlers.ClockHandler.rebootSystem = function( restart )
    {
       /** @type {tsa.cmd.Command} */ var cmd = null;
       
       cmd = new Command( { action              : 'clock/ajaxReboot'
                          , clearSystemMessages : false
                          , parameters          : { "reboot" : restart } 
                          } );
       cmd.execute();
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.ClockHandler;
    
    //-------------------------------------------------------------------------
  }
); 