/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ShowCommand
          , ButtonFactory
          , ContentArea
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'ConnectivityHandler' );
    var i18n   = tsa.i18n;

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.ConnectivityHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.ConnectivityHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ConnectivityHandler.prototype ).constructor = tsa.cmd.handlers.ConnectivityHandler;

    //-------------------------------------------------------------------------

    /**
     * Show main screen.
     */
    tsa.cmd.handlers.ConnectivityHandler.prototype._index = function()
    {
    	/** @type {Object}                    */ var i18n         			 = tsa.i18n;
    	/** @type {tsa.widgets.ContentArea}   */ var contentArea        	 = null;
    	/** @type {tsa.widgets.FormPanel}     */ var formPanelAccess    	 = null;
    	/** @type {tsa.widgets.FormPanel}     */ var formPanelSettings    	 = null;
    	/** @type {tsa.widgets.FormPanel}     */ var formPanelAuthentication = null;
    	/** @type {tsa.widgets.FormPanel}     */ var formPanelTest 			 = null;
    	/** @type {tsa.widgets.ButtonFactory} */ var buttonContainer   		 = ButtonFactory.buildButtonContainer();
    	/** @type {String} 				      */ var imagePath         		 = tsa.attributes.imagesPath + '/';
    	/** @type {tsa.cmd.Command}           */ var getConnectivityCmd  	 = new Command( { action : 'connectivity/getConnectivity', clearSystemMessages : false } );
    	/** @type {tsa.cmd.ShowCommand}       */ var showCmdCancel           = new ShowCommand( { action : 'main/index' } );
        /** @type {tsa.cmd.ShowCommand}       */ var showCmdSave             = new ShowCommand( { action : 'connectivity' , clearSystemMessages : false } );
        /** @type {Element}                   */ var saveButton    			 = null;
    	/** @type {Element}                   */ var testButton    			 = null;
    	/** @type {Element}                   */ var cancelButton			 = null;
    	/** @type {Element}                   */ var elTestConnectionDiv 	 = null;
    	/** @type {Element}                   */ var elAccess = document.createElement( 'select' );
    	/** @type {Element}                   */ var elTestConnectionResultsDiv = null;


    	// Function to disable inputs depending on access/connection settings
    	/** @type {function(string)} */ var disableInputs = function( connectionType )
    	{
    		if(connectionType === 'proxy' )
    		{
    			$( '[name="address"]' ).prop('disabled' , false );
    			$( '[name="port"]'    ).prop('disabled' , false );
    			$( '[name="user"]'    ).prop('disabled' , true  );
    			$( '[name="password"]').prop('disabled' , true  );
    			$( '[name="password_confirm"]').prop('disabled' , true  );
    		}
    		else if(connectionType === 'authproxy' )
    		{
    			$( '[name="address"]' ).prop('disabled' , false  );
    			$( '[name="port"]'  ).prop('disabled' , false  );
    			$( '[name="user"]'    ).prop('disabled' , false  );
    			$( '[name="password"]').prop('disabled' , false  );
    			$( '[name="password_confirm"]').prop('disabled' , false  );
    		}
    		else{
    			$( '[name="address"]' ).prop('disabled' , true  );
    			$( '[name="port"]'  ).prop('disabled' , true  );
    			$( '[name="user"]'    ).prop('disabled' , true  );
    			$( '[name="password"]').prop('disabled' , true  );
    			$( '[name="password_confirm"]').prop('disabled' , true  );
    		}
    	};

    	// Initialize the content area (its title and its help document)...
    	contentArea = new ContentArea ( { title   : i18n.labelIbmConnHeading
    		, helpDoc : 'sshelp_ibm_connectivity_panel.html' } );

    	contentArea.appendMessage( { text : i18n.labelIbmConnCaption } );



    	//ACCESS PANEL
    	formPanelAccess = new FormPanel( { title  : i18n.labelIbmConnAccessHeading
    		, action : 'connectivity/ajaxUpdateConnectivity' } );
    	formPanelAccess.setDescription( i18n.labelIbmConnAccessDesc );


    	//Add selection for connection type
    	elAccess.name = "access";
    	elAccess.id   = "access";

    	$.each( [ { value: 'direct', text: i18n.labelIbmConnAccessDirect   }
    	, { value: 'proxy' , text: i18n.labelIbmConnAccessProxy }
    	, { value: 'authproxy' , text: i18n.labelIbmConnAccessProxyAuth } ]
    	, function( optionDataIndex, optionData )
    	{
    		var opt = document.createElement( 'option' );

    		opt.value = optionData.value;
    		$( opt ).text( optionData.text );
    		$( elAccess ).append( opt );
    	}
    	);

    	$( elAccess ).on( 'change', function()
    	{
    		disableInputs( this.value );
    	} );

    	formPanelAccess.appendFieldRow( { label    : i18n.labelIbmConnAccess
    		, required : true
    		, element  : elAccess
    	} );


    	//SSL PROXY SETTINGS PANEL
    	formPanelSettings = new FormPanel( { title : i18n.labelIbmConnProxyHeading} );
    	formPanelSettings.setDescription( i18n.labelIbmConnProxyDesc );

    	formPanelSettings.appendFieldRow( { label : i18n.labelIbmConnProxyAddress
    		, required : true
    		, hint     : i18n.labelIbmConnProxyAddressDesc
    		, element  : $( '<input type="text" size="50" name="address" id="address" value="" required/>' ).get(0)
    	} );

    	formPanelSettings.appendFieldRow( { label : i18n.labelIbmConnProxyPort
    		, required : true
    		, hint     : i18n.labelIbmConnProxyPortDesc
    		, element  : $( '<input type="text" size="50" name="port" id="port" value="" required/>' ).get(0)
    	} );


    	//SSL PROXY AUTHENTICATION PANEL
    	formPanelAuthentication = new FormPanel( { title  : i18n.labelIbmConnProxyAuthHeading} );
    	formPanelAuthentication.setDescription( i18n.labelIbmConnProxyAuthDesc );

    	formPanelAuthentication.appendFieldRow( { label : i18n.labelIbmConnProxyAuthUser
    		, required : true
    		, hint     : i18n.labelIbmConnProxyAuthUserDesc
    		, element  : $( '<input type="text" size="50" name="user" id="user" value="" required/>' ).get(0)
    	} );

    	formPanelAuthentication.appendFieldRow( { label : i18n.labelIbmConnProxyAuthPassw
    		, required : true
    		, hint     : i18n.labelIbmConnProxyAuthPasswDesc
    		, element  : $( '<input type="password" size="50" name="password" id="password" value="" required/>' ).get(0)
    	} );

    	formPanelAuthentication.appendFieldRow( { label : i18n.labelIbmConnProxyAuthPassw2
    		, required : true
    		, element  : $( '<input type="password" size="50" name="password_confirm" id="password_confirm" value="" required/>' ).get(0)
    	} );

    	//TEST CONNECTION PANEL
    	formPanelTest = new FormPanel( { title  : i18n.labelIbmConnTestConnHeading} );
    	formPanelTest.hide();

    	elTestConnectionDiv = document.createElement( 'div' );
    	$( elTestConnectionDiv ).attr( 'id',  'connectivityPanel' );
    	$( elTestConnectionDiv ).attr( 'role' , 'region' );
    	$( elTestConnectionDiv ).attr( 'aria-live', 'polite' );
    	$( elTestConnectionDiv ).attr( 'aria-labelledby', 'connectivityData' );
    	$( elTestConnectionDiv ).addClass( 'ibm-container ibm-show-hide' );

      	var elTestConnectionParag = document.createElement( 'p' );
    	$( elTestConnectionParag ).css({ 'text-align': 'center' });

    	var elTestConnectionImg = document.createElement( 'img' );
    	$( elTestConnectionImg ).attr( 'src', imagePath + i18n.imageLoader16 );
    	$( elTestConnectionImg ).attr( 'title', i18n.labelRunningConnTestTooltip )
    	$( elTestConnectionImg ).css({ 'width': '16', 'height': '16', 'vertical-align' : 'middle' });

    	var elTestConnectionSpan = document.createElement( 'span' );
    	$( elTestConnectionSpan ).css({'color': '#66666' });
    	$( elTestConnectionSpan ).text( i18n.labelPleaseWait );

    	var elTestConnectionBr = document.createElement( 'br' );

    	$( elTestConnectionParag ).append( elTestConnectionImg );
    	$( elTestConnectionParag ).append( elTestConnectionBr );
    	$( elTestConnectionParag ).append( elTestConnectionSpan );


    	$( elTestConnectionDiv ).append( elTestConnectionParag );
    	formPanelTest.appendJointRow( elTestConnectionDiv );

    	//link forms
    	formPanelAccess.linkTo( formPanelSettings );
    	formPanelSettings.linkTo( formPanelAuthentication );
    	formPanelAuthentication.linkTo( formPanelTest );

    	//Save button
    	saveButton = ButtonFactory.buildButton( { type : 'arrow' , grayout : false , label : i18n.buttonSave
    		, onclick : function( event ){
    			formPanelAccess.submit();
    		}
    	});

    	$( formPanelAccess ).on( 'executionSucceeded', function( event ){
    			( new ShowCommand( { action : 'connectivity/index' , clearSystemMessages : false } ) ).execute();
    	} );


    	$( formPanelAccess ).on( 'executionFailed', function( event ){
    		 if( TV.typeIsObject( typeof event.response.payload.errors ) )
    		 {
    	       for (var fieldName in event.response.payload.errors )
    	       {
    	       	  formPanelAccess.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
    	       }
    	     }
    	} );


    	//Test button
    	testButton = ButtonFactory.buildButton( { type : 'arrow' , grayout : false, label : i18n.buttonTestConnection
    		, title : i18n.labelIbmConnSaveWarning
    		, onclick : function( event ){
    			var testCmd = new Command( { action : 'connectivity/ajaxTestConnectivity', clearSystemMessages : false,  blockUI : false });

    			//show test form
    			formPanelTest.show();
    			$( elTestConnectionParag ).css( { 'text-align': 'center' });
    			$( elTestConnectionImg   ).show();
				$( elTestConnectionSpan  ).show();
				$( elTestConnectionBr    ).show();

				if ( elTestConnectionResultsDiv !== null ){
					$( elTestConnectionResultsDiv ).empty();
				}

				//execute test
				testCmd.execute();

				//process results
	    		$( testCmd ).on( 'executionSucceeded', function( event ) {

				//remove 'Please wait' image and text
				$( elTestConnectionImg     ).hide();
				$( elTestConnectionSpan    ).hide();
				$( elTestConnectionBr      ).hide();
				$( elTestConnectionParag   ).css( { 'text-align': 'left' } );
				elTestConnectionResultsDiv = document.createElement( 'div' );

				var testStatus = event.response.payload.connTestResult;
				var logId = event.response.payload.auditLogId;
				var results = event.response.payload.results;

				//append status message
				$( elTestConnectionResultsDiv ).css( {'font-size' : '12px', 'font-style': 'italic' } );
				$( elTestConnectionResultsDiv ).append( document.createTextNode( testStatus + " ") );

				//append log link
				 if( !TV.typeIsUndefined( typeof logId ) && logId !== null ){
					var logLink  = document.createElement( 'a' );
					logLink.href = Command.URL_CONTROLLER_BASE + 'logs/show/'+logId;
					$( logLink ).addClass( 'ibm-feature-link' );
  		          	$( logLink ).text( i18n.labelConnTestAuditLog );
  		          	$( elTestConnectionResultsDiv ).append( logLink );
  		            $( elTestConnectionParag ).append( elTestConnectionResultsDiv );
  		        }
		    });
    	  }
    	});


    	//Cancel button
    	cancelButton = ButtonFactory.buildButton({ type : 'cancel', grayout : false , label : i18n.buttonCancel
    		, onclick :  function( event ) {
    			( new ShowCommand( { action : 'main/index' })).execute();
    		  }
    	});

    	$( buttonContainer ).append( saveButton );
    	$( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
    	$( buttonContainer ).append( testButton );
    	$( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
    	$( buttonContainer ).append( cancelButton );

    	//Append forms and buttons
    	contentArea.append( formPanelAccess.getDomNode() );
    	contentArea.append( buttonContainer );

    	// Get current connectivity information
    	$( getConnectivityCmd ).on( 'executionSucceeded', function( event ) {
    		for( var key in event.response.payload ) {
    			$( '[name="' + key + '"]' ).val( event.response.payload[ key ] );
    		}

    		//enable/disable text boxes
    		disableInputs( event.response.payload.access );
    	} );

    	getConnectivityCmd.execute();

    	this.show( contentArea.getDomNode() );
    };

    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.ConnectivityHandler;
  }
);