/*
 * IBM Technical Support Appliance Version 2 Release 5 Modification 0
 *
 * (C) Copyright IBM Corp. 2019
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ShowCommand
          , ButtonFactory
          , ContentArea
          , FormPanel
          , IconFactory
          , TablePanel )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'CredentialshmcHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.CredentialshmcHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );

      // Initializing members...
      this.scopeSets = {};
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.CredentialshmcHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.CredentialshmcHandler.prototype ).constructor = tsa.cmd.handlers.CredentialshmcHandler;

    //-------------------------------------------------------------------------
    /**
     * Show main screen (list the credentials).
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype._index = function()
    {

    };
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.CredentialshmcHandler.prototype._new = function()
    {
        /** @type {string}                  */ var pid             = this.getParameter('pid');
        /** @type {Object}                  */ var i18n            = tsa.i18n;
        /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title   : i18n.labelDiscoHMCCredsNewCredsHeading
                                                                                      , helpDoc : 'sshelp_new_hmc_discovery_credential_panel.html' } );
        /** @type {tsa.widgets.FormPanel}   */ var credForm        = this.createCredentialForm();
        /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();

        credForm.setActionAttribute( 'credentials/postCredential' );
        
        // Buttons...
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                                , label   : i18n.buttonSave
                                                                , onclick : function( event ) {
                                                                    var inputValue = $("input[name='name']").val();
                                                                    if (inputValue.length==0) {
                                                                        credForm.validate();
                                                                    } else { 
                                                                      $("input[name='name']").val(inputValue + "_HMCCrt_");
                                                                      credForm.submit();
                                                                    }
                                                                    } } ) );
        // Remove extension "_HMCCrt_" when execution failed
        $( credForm ).on( 'executionFailed', function( event )
        {
          var inputValue = $("input[name='name']").val();
          $("input[name='name']").val(inputValue.replace("_HMCCrt_",""));
        } );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'cancel'
                                                                , label   : i18n.buttonCancel
                                                                , onclick : function( event ) { ( new ShowCommand( { action : 'scopesetshmc/index' } ) ).execute(); } } ) );

        // Appending content...
        contentArea.append( credForm.getDomNode() );
        contentArea.append( buttonContainer );
        this.show( contentArea.getDomNode() );

        // What happens when creation succeeds? Print success message, then show the credentials list.
        $( credForm ).on( 'executionSucceeded', function( event )
        {
          /** @type {string} */ var credentialName = $( credForm.getDomNode() ).find( '[name=\'name\']' ).val();

          tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoHMCCredsCreated, credentialName.replace("_HMCCrt_","") ) } );
          
          ( new ShowCommand( { action              : 'scopesetshmc/show'
              , clearSystemMessages : false
              , parameters : { pid : pid}} ) ).execute();
          
        } );
        $( credForm ).on( 'executionFailed', function( event )
            {
            if ( TV.typeIsObject( typeof event.response.payload.errors ) ) 
            {
              for ( /** @type {string} */ var fieldName in event.response.payload.errors ) {
                credForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
            }
              delete event.response.payload.errors;
            }
            } );
    };
    //-------------------------------------------------------------------------
      tsa.cmd.handlers.CredentialshmcHandler.prototype._show = function(){

        /** @type {Object}                  */ var i18n            = tsa.i18n;
        /** @type {string}                  */ var pid             = this.getParameter( 'entityId' );
        /** @type {string}                  */ var sepid           = this.getParameter( 'pid' );
        /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title   : i18n.labelDiscoHMCCredsHeading
                                                                                      , helpDoc : 'sshelp_discovery_credential_panel.html' } );
        /** @type {tsa.widgets.TablePanel}  */ var generalTable    = new TablePanel( { title   : i18n.labelDiscoHMCCredsGeneralHeading
                                                                                     , columns : [ { id: 'name'  , label: '' , width: '30%' , sortable: false }
                                                                                                 , { id: 'value' , label: '' , width: '70%' , sortable: false } ] } );
        /** @type {tsa.widgets.TablePanel}  */ var propsTable      = new TablePanel( { title   : i18n.labelDiscoHMCCredsPropsHeading
                                                                                     , columns : [ { id: 'name'  , label: i18n.labelDiscoHMCCredsPropsName  , width: '40%' , sortable: false }
                                                                                                 , { id: 'value' , label: i18n.labelDiscoHMCCredsPropsValue , width: '60%' , sortable: false } ] } );
        /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
        
        /*type funciton*/
        var i18nAuthType = function ( authtype ) {
            /** @type {string} */ var i18nLabel = '';

            if( !TV.typeIsString( typeof authtype ) )
            {
              authtype = '' + authtype;
            }

            switch( authtype )
            {
              case 'authType_default'    : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypeDefault   ; break;
              case 'authType_password'   : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypePassw     ; break;
              case 'authType_pki'        : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypePki       ; break;
              case 'authType_pki_scoped' : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypePkiScoped ; break;
              default                    : i18nLabel = authtype;
            }

            return i18nLabel;
        };
        

        var getCredentialCmd = new Command( { action              : 'credentials/getCredential'
                                            , parameters          : { 'pid' : pid }
                                            , clearSystemMessages : false } );

        $( getCredentialCmd ).on( 'executionSucceeded', function( event )
        {
          /** @type {Object} */ var credential = event.response.payload.credential;

          if( TV.typeIsObject( typeof credential ) && ( credential !== null ) )
          {
            /** @type {Element} */ var scopeSetSpan      = document.createElement( 'span' );
            /** @type {?string} */ var userName          = credential.userName;
            /** @type {?string} */ var scopeSetName      = credential.scopeSetName;
            /** @type {boolean} */ var restricted        = ( TV.typeIsString( typeof scopeSetName ) && ( scopeSetName.length > 0 ) );
            /** @type {Object}  */ var scopeSet          = ( ( restricted && TV.typeIsObject( typeof credential.scopeSet ) ) ? credential.scopeSet : null );
            /** @type {boolean} */ var scopeSetHasSSHKey = ( ( scopeSet !== null ) && ( scopeSet.hasSSHKey === true ) );

            if( userName     === null ) userName     = '';
            if( scopeSetName === null ) scopeSetName = '';

            $( scopeSetSpan ).text( scopeSetName.replace("_HMCCrt_","") );
            $( scopeSetSpan ).css( { 'color' : ( credential.scopeSetMissing ? 'red' : 'black' ) } );

            generalTable.appendRow( [ i18n.labelDiscoHMCCredsGeneralName     , credential.name.replace("_HMCCrt_","")           ] );
            generalTable.appendRow( [ i18n.labelDiscoHMCCredsGeneralType     , credential.authClassLabel ] );
            generalTable.appendRow( [ i18n.labelDiscoHMCCredsGeneralUserName , userName                  ] );
            generalTable.appendRow( [ i18n.labelDiscoHMCCredsGeneralScopeSet , scopeSetSpan              ] );

            if( tsa.isDebugModeOn() )
            {
              generalTable.appendRow( [ i18n.labelDiscoHMCCredsGeneralPidDebug   , credential.pid           ] );
              generalTable.appendRow( [ i18n.labelDiscoHMCCredsGeneralClassDebug , credential.authClassName ] );
            }

            if( TV.typeIsObject( typeof credential.properties ) && ( credential.properties !== null ) )
            {
              for( /** @type {string} */ var propertyName in credential.properties )
              {
                /** @type {string} */ var propertyValue = credential.properties[ propertyName ];

                if( propertyName === 'authtype' )
                {
                  /** @type {boolean} */ var isPKI = ( propertyValue.indexOf( 'authType_pki' ) === 0 );

                  if( isPKI )
                  {
                      propertyValue  = i18nAuthType( propertyValue );
                  }
                  
                }
                if(propertyName!=='order')
                {
                  //improve the replace with a specficic
                  propsTable.appendRow( [ propertyName, propertyValue.replace("_HMCCrt_","") ] );
                }

              }
            }
          }

          $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'back'
                                                                  , grayout : true
                                                                  , label   : i18n.buttonGoBack
                                                                  , onclick : function( event ) { ( new ShowCommand( { 
                                                                                                                      action : 'scopesetshmc/show'
                                                                                                                     , parameters : { pid : sepid   }
                                                                                                                          } ) ).execute(); 
                                                                                                                     } } ) );
          $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
          $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                                  , label   : i18n.buttonDiscoCredsEdit
                                                                  , onclick : function( event ) { tsa.cmd.handlers.CredentialshmcHandler.prototype.editCredentialClicked( credential.pid, sepid ); } } ) );

          contentArea.append( generalTable.getDomNode() );
          contentArea.append( propsTable.getDomNode() );
          contentArea.append( buttonContainer );
        } );

        getCredentialCmd.execute();

        this.show( contentArea.getDomNode() );

      };

      tsa.cmd.handlers.CredentialshmcHandler.prototype._edit = function(){

        /** @type {Object}                  */ var i18n            = tsa.i18n;
        /** @type {String}                  */ var credentialId    = this.getParameter( 'entityId' );
        /** @type {String}                  */ var pid             = this.getParameter( 'pid' );
        /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title         : i18n.labelDiscoHMCEditCredsHeading
                                                                                      , helpDoc       : 'sshelp_discovery_hmc_credentials_edit_panel.html' } );
        /** @type {tsa.widgets.FormPanel}   */ var credForm        = null;
        /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
        /** @type {tsa.cmd.Command}         */ var getCredCmd      = new Command( { action              : 'credentials/getCredential'
                                                                                  , parameters          : { 'pid' : credentialId }
                                                                                  , clearSystemMessages : false } );
        $( getCredCmd ).on( 'executionSucceeded', $.proxy( function( event )
        {
          credForm = this.createCredentialForm( event.response.payload.credential );
          credForm.setActionAttribute( 'credentials/putCredential' );

          // What happens when edition succeeds? Print success message, then show the credentials list.
          $( credForm ).on( 'executionSucceeded', function( event )
          {
            /** @type {string} */ var credentialName = $( credForm.getDomNode() ).find( '[name=\'name\']' ).val();

            tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoHMCCredsUpdated, credentialName.replace("_HMCCrt_","") ) } );

            ( new ShowCommand( { action              : 'scopesetshmc/show'
                                 , parameters          : { pid : pid }    
                               , clearSystemMessages : false } ) ).execute();
          } );

          // Appending content...
          contentArea.append( credForm.getDomNode() );
          contentArea.append( buttonContainer );
        }
        , this ) );

        // Buttons...
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                                , label   : i18n.buttonSave
                                                                , onclick : function( event ) { credForm.submit(); } } ) );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'cancel'
                                                                , label   : i18n.buttonCancel
                                                                , onclick : function( event ) { ( new ShowCommand( { 
                                                                                                                      action : 'scopesetshmc/show'
                                                                                                                    , parameters : { pid : pid}
                                                                                                                    } ) ).execute(); 
                                                                                                                    } } ) );

        // Show content area now...
        this.show( contentArea.getDomNode() );

        // Get the credential...
        getCredCmd.execute();

      }
      /**
       * Edit credential - event handler.
       * @param {string} credentialPID The credential PID.
       */

      tsa.cmd.handlers.CredentialshmcHandler.prototype.editCredentialClicked = function( credentialPID , pid)
      {
        ( new ShowCommand( { action : 'credentialshmc/edit/' + credentialPID + '/'
                            , parameters : { pid : pid } } ) ).execute();
      };
      //-------------------------------------------------------------------------
      tsa.cmd.handlers.CredentialshmcHandler.prototype.createCredentialForm = function( credential )
      {
        /** @type {Object}                */ var i18n            = tsa.i18n;
        /** @type {boolean}               */ var isEditionMode   = TV.typeIsObject( typeof credential ) && ( credential !== null );
        /** @type {tsa.widgets.FormPanel} */ var generalForm     = null;
        /** @type {tsa.widgets.FormPanel} */ var nameForm        = new FormPanel( { title       : i18n.labelDiscoHMCCredsNameHeading
                                                                                  , description : i18n.labelDiscoHMCCredsNameDesc } );
        /** @type {tsa.widgets.FormPanel} */ var scopeSetForm    = new FormPanel( { title       : i18n.labelDiscoHMCCredsScopeSetHeading
                                                                                  , description : i18n.labelDiscoHMCCredsScopeSetDesc } );
        /** @type {tsa.widgets.FormPanel} */ var hostAuthForm    = new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         
                                                                                  , description: i18n.labelDiscoCredsHostAuthEnterDesc} );
        /** @type {string}                */ var pid             = this.getParameter('pid');
        
        hostAuthForm.putHiddenInput("RestrictionType","limit");
        hostAuthForm.putHiddenInput( 'ClassName', 'com.collation.platform.security.auth.HostAuth' );
        hostAuthForm.putHiddenInput("CredentialType","HostAuth");
        /** @type {tsa.widgets.FormPanel} */ var firstForm       = nameForm;
        /** @type {tsa.cmd.Command}       */ var getScopeSetsCmd = new Command( { action              : 'scopesets/getScopeSets'
                                                                                , parameters          : { 'withCredentials' : 'true' 
                                                                                                         , 'filter'          : 'HMC' }
                                                                                , clearSystemMessages : false } );

        if (isEditionMode) {
            nameForm.linkTo( hostAuthForm );
            nameForm.putHiddenInput("scopename", credential.scopeSet.pid);
        } else {
          // Linking all forms together when new
          nameForm.linkTo( scopeSetForm );
          scopeSetForm.linkTo( hostAuthForm );
        }

        // Fields (common):
        /** @type {Element} */ var nameField     = $( '<input type="text" id="name" name="name" size="50" maxlength="250" />' ).get(0);
        
        /** @type {Element} */ var scopeSetField = $( '<select name="scopename" id="scopename"></select>' ).get(0);
        
        
        

        

        nameForm        .appendFieldRow( { label: i18n.labelDiscoCredsName          , required: true  , element: nameField     } );
        
        
        scopeSetForm    .appendFieldRow( { label: i18n.labelDiscoCredsScopeSetName  , required: true  , element: scopeSetField } );


        if( isEditionMode )
        {
          /** @type {Element} */ var nameSpan = document.createElement( 'span' );
          /** @type {Element} */ var typeSpan = document.createElement( 'span' );

          generalForm = new FormPanel( { title: i18n.labelDiscoCredsGeneralHeading } );

          $( nameSpan ).css( { 'font-weight' : 'bold' } );
          $( nameSpan ).text( credential.name.replace("_HMCCrt_",""));
          $( typeSpan ).text( credential.authClassLabel );

          generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralName, element: nameSpan } );
          generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralType, element: typeSpan } );

          generalForm.linkTo( nameForm );
          firstForm = generalForm;

          firstForm.putHiddenInput( 'pid'            , credential.pid                 );
          firstForm.putHiddenInput( 'name'           , credential.name                );
          

          $( nameField ).val( credential.name                );
          

          nameForm.hide();
          

          
        }

        /*type funciton*/
        var i18nAuthType= function ( authtype ) {
            /** @type {string} */ var i18nLabel = '';

            if( !TV.typeIsString( typeof authtype ) )
            {
              authtype = '' + authtype;
            }

            switch( authtype )
            {
              case 'authType_default'    : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypeDefault   ; break;
              case 'authType_password'   : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypePassw     ; break;
              case 'authType_pki'        : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypePki       ; break;
              case 'authType_pki_scoped' : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypePkiScoped ; break;
              default                    : i18nLabel = authtype;
            }

            return i18nLabel;
        };

        // Fields for HostAuth:
        /** @type {Element} */ var hostAuthUserName      = $( '<input type="text"   id="HostAuth_username"  name="HostAuth_username"         size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var hostAuthPassword1     = $( '<input type="password" id="HostAuth_password" name="HostAuth_password"         size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var hostAuthPassword2     = $( '<input type="password" id="HostAuth_password_confirm" name="HostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var hostAuthAuthType      = $( '<select name="HostAuth_authtype" id="HostAuth_authtype" >'
                                                            +   '<option value="authType_default" selected="selected"' + '>' + i18nAuthType( 'authType_default'    ) + '</option>'
                                                            +   '<option value="authType_password"'                    + '>' + i18nAuthType( 'authType_password'   ) + '</option>'
                                                            +   '<option  value="authType_pki_scoped"'                  + '>' + i18nAuthType( 'authType_pki_scoped' ) + '</option>'
                                                            +   '<option  value="authType_pki"'                         + '>' + i18nAuthType( 'authType_pki'        ) + '</option>'
                                                            + '</select>' ).get(0);
        /** @type {Element} */ var hostAuthUploadKey     = $( '<input type="file" id="HostAuth_sshKey" name="HostAuth_sshKey" />' ).get(0);
        /** @type {Element} */ var hostAuthUploadKeyType = document.createElement( 'p' );
        /** @type {Element} */ var hostAuthUploadKeyWarn = document.createElement( 'p' );
        /** @type {Element} */ var hostAuthUploadKeyDiv  = document.createElement( 'div' );

        $( hostAuthUploadKeyType ).text( i18n.labelDiscoCredsInternalSSHKey );

        $( hostAuthUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
        $( hostAuthUploadKeyWarn ).css( { 'margin-top'       : '0.80em'
                                        , 'border'           : '1px solid #c0c000'
                                        , 'border-radius'    : '5px'
                                        , 'background-color' : '#ffffc0' } );

        $( hostAuthUploadKeyDiv ).append( hostAuthUploadKeyType );
        $( hostAuthUploadKeyDiv ).append( hostAuthUploadKey     );
        $( hostAuthUploadKeyDiv ).append( hostAuthUploadKeyWarn );

        hostAuthForm.appendFieldRow( { label: i18n.labelDiscoCredsUser     , required: true  , element: hostAuthUserName     } );
        hostAuthForm.appendFieldRow( { label: i18n.labelDiscoCredsPassw1   , required: false , element: hostAuthPassword1    } );
        hostAuthForm.appendFieldRow( { label: i18n.labelDiscoCredsPassw2   , required: false , element: hostAuthPassword2    } );
        hostAuthForm.appendFieldRow( { label: i18n.labelDiscoCredsAuthType , required: false , element: hostAuthAuthType     } );
        hostAuthForm.appendFieldRow( { label: i18n.labelDiscoCredsSSHKey   , required: true  , element: hostAuthUploadKeyDiv } );
    // Common value initialization for fields of HostAuth and WindowsHostAuth:
        if( isEditionMode )
        {
          /** @type {boolean} */ var isHostAuth    = ( credential.authClassSimpleName === 'HostAuth'        );

          if( isHostAuth)
          {
            /** @type {Element} */ var usernameField     =   hostAuthUserName ;
            /** @type {Element} */ var authtypeField     =  hostAuthAuthType ;
            /** @type {boolean} */ var isPKI             = ( credential.properties.authtype.indexOf( 'authType_pki' ) === 0 );
            /** @type {boolean} */ var restricted        = ( TV.typeIsString( typeof credential.scopeSetName ) &&
                                                             ( credential.scopeSetName.length > 0 ) );
            /** @type {Object}  */ var scopeSet          = ( ( restricted && TV.typeIsObject( typeof credential.scopeSet ) ) ? credential.scopeSet : null );
            /** @type {string}  */ var authType          = credential.properties.authtype;
            /** @type {boolean} */ var scopeSetHasSSHKey = ( ( scopeSet !== null ) && ( scopeSet.hasSSHKey === true ) );


            $( usernameField ).val( credential.properties.username );
            $( authtypeField ).val( isHostAuth ? authType : /* Windows only supports: */ 'authType_password' );
          }
        }

        // Event handler - scope set restriction type changed:
        var onRestrictionTypeFieldChanged = $.proxy( function( event )
        {
          /*type function */
          var  retrievePKICredentialsFor = function( scopeSetData )
            {
              /** @type {Array.<string>} */ var pkiCredentials = [];

              if( TV.typeIsObject( typeof scopeSetData             ) && ( scopeSetData !== null ) &&
                  TV.typeIsObject( typeof scopeSetData.credentials ) && TV.isInstanceOf( scopeSetData.credentials, Array ) )
              {
                $.each( scopeSetData.credentials, function( credIndex, cred )
                {
                  if( TV.typeIsObject( typeof cred                     ) && ( cred                     !== null           ) &&
                      TV.typeIsObject( typeof cred.properties          ) && ( cred.properties          !== null           ) &&
                      TV.typeIsString( typeof cred.properties.authtype ) && ( cred.properties.authtype === 'authType_pki' ) )
                  {
                    pkiCredentials.push( cred );
                  }
                } );
              }

              return pkiCredentials;
            };


          /** @type {boolean}        */ var isHostAuth          = !hostAuthForm.isHidden();
          /** @type {boolean}        */ var authSupportsPKI     =  isHostAuth;
          /** @type {Element}        */ var authTypeField       =  hostAuthAuthType  ;
          /** @type {Element}        */ var warnMsgElement      = hostAuthUploadKeyWarn ;
          /** @type {boolean}        */ var pkiGlobal           = ( authSupportsPKI && ( $( authTypeField ).val() === 'authType_pki'        ) );
          /** @type {boolean}        */ var pkiScoped           = ( authSupportsPKI && ( $( authTypeField ).val() === 'authType_pki_scoped' ) );
          /** @type {boolean}        */ var isPKI               = ( authSupportsPKI && ( pkiGlobal || pkiScoped ) );
          /** @type {boolean}        */ var restricted          = true;
          /** @type {?string}        */ var selectedScopeSetPID = ( ( restricted && ( scopeSetField.options.length > 0 ) ) ? $( scopeSetField ).val() : null );
          /** @type {Object}         */ var scopeSet            = ( ( TV.typeIsString( typeof selectedScopeSetPID                   ) &&
                                                                      TV.typeIsObject( typeof this.scopeSets[ selectedScopeSetPID ] ) )
                                                                    ? this.scopeSets[ selectedScopeSetPID ] : null );
          /** @type {Array.<Object>} */ var scopeSetPkiCreds    = ( ( isPKI && ( scopeSet !== null ) ) ? retrievePKICredentialsFor( scopeSet ) : [] );
          /** @type {boolean}        */ var scopeSetHasPkiCreds = ( ( scopeSetPkiCreds !== null ) && ( scopeSetPkiCreds.length > 0 ) );
          /** @type {string}         */ var pkiCredsAsString    = '';
          /** @type {boolean}        */ var showWarningMessage  = ( isPKI && scopeSetHasPkiCreds && ( !isEditionMode || ( selectedScopeSetPID !== credential.scopeSetPID ) ) );

          $( warnMsgElement ).empty();
          $( warnMsgElement )[ showWarningMessage ? 'show' : 'hide' ]();

          if( authSupportsPKI )
          {
            if( scopeSetHasPkiCreds )
            {
              /** @type {string} */ var warnMsg = '';

              $.each( scopeSetPkiCreds, function( credIndex, cred )
              {
                pkiCredsAsString += ( ( credIndex > 0 ) ? ', ' : '' ) + cred.name;
              } );

              warnMsg = tsa.bind( i18n.labelDiscoCredsPKICredExists, [ pkiCredsAsString, scopeSet.name ] );

              $( warnMsgElement ).append( IconFactory.buildSeverityIcon( { severity : 'WARN' } )
                                        , document.createTextNode( '\xa0' + warnMsg ) );
            }
            
          }
          scopeSetField.disabled = !restricted;
          
        }
        , this );

        
        $( scopeSetField ).on( 'change', onRestrictionTypeFieldChanged );

        // Event handler - HostAuth and WindowHostAuth AuthType changed:
        var onHostAuthAuthTypeChanged = function( event )
        {
          /** @type {boolean} */ var isHostAuth          = !hostAuthForm.isHidden();
          /** @type {Element} */ var authTypeField       = hostAuthAuthType ;
          /** @type {Element} */ var keyTypeMessage      = hostAuthUploadKeyType ;
          /** @type {Element} */ var uploadKeyField      = hostAuthUploadKey ;
          /** @type {boolean} */ var pkiGlobal           = ( $( authTypeField ).val() === 'authType_pki'        );
          /** @type {boolean} */ var pkiScoped           = ( $( authTypeField ).val() === 'authType_pki_scoped' );
          /** @type {boolean} */ var isPKI               = (pkiGlobal || pkiScoped);
          /** @type {boolean} */ var hideSSHKeyRow       = !isPKI;
          /** @type {boolean} */ var hideSSHKeyUpload    = ( hideSSHKeyRow || pkiGlobal || isEditionMode );
          /** @type {boolean} */ var disableSSHKeyUpload = !pkiScoped;
          /** @type {boolean} */ var hideKeyTypeMessage  = pkiScoped && !isEditionMode;
          /** @type {Element} */ var replaceButton       = document.createElement( 'button' );
          /** @type {Element} */ var row                 = $( uploadKeyField ).closest( 'tr' ).get(0);

          $( row            )[ hideSSHKeyRow      ? 'hide' : 'show' ]();
          $( uploadKeyField )[ hideSSHKeyUpload   ? 'hide' : 'show' ]();
          $( keyTypeMessage )[ hideKeyTypeMessage ? 'hide' : 'show' ]();

          // Password or Passphare? use the right label...
          //$( hostAuthPassword1        ).closest( 'tr' ).find( 'label' ).text( isPKI ? i18n.labelDiscoCredsSSlPassw1 : i18n.labelDiscoCredsPassw1 );
          /** @type {string } */ var labelValue = isPKI ? i18n.labelDiscoHMCCredsSSlPassw1 : i18n.labelDiscoHMCCredsPassw1 ;
          /** @type {Element} */ var labelNode  = $( hostAuthPassword1        ).closest( 'tr' ).find( 'label' );
          labelNode.empty();
          labelNode.append(labelValue,$("<span class='ibm-required'> *</span>").get(0));
          
          //$( hostAuthPassword2).closest( 'tr' ).find( 'label' ).text( isPKI ? i18n.labelDiscoCredsSSlPassw2 : i18n.labelDiscoCredsPassw2 );
          labelValue = isPKI ? i18n.labelDiscoHMCCredsSSlPassw2 : i18n.labelDiscoHMCCredsPassw2 ;
          labelNode  = $( hostAuthPassword2        ).closest( 'tr' ).find( 'label' );
          labelNode.empty();
          labelNode.append(labelValue, $("<span class='ibm-required'> *</span>").get(0));

          if( isPKI )
          {
            $( keyTypeMessage ).text( pkiGlobal ? i18n.labelDiscoCredsInternalSSHKey
                                                : i18n.labelDiscoCredsExternalSSHKey );
            if( pkiScoped )
            {
              
              if( isEditionMode )
              {
                replaceButton.type = 'button';
                $( replaceButton ).text( i18n.labelDiscoCredsExternalSSHKeyReplace );
                $( replaceButton ).on( 'click', function( event )
                                                {
                                                  $( keyTypeMessage ).hide();
                                                  $( uploadKeyField ).show();
                                                } );

                $( keyTypeMessage ).append( document.createTextNode( /* nbsp */ '\xa0' ) );
                $( keyTypeMessage ).append( replaceButton );
              }
            }
          }
          
          onRestrictionTypeFieldChanged();
          uploadKeyField.disabled = disableSSHKeyUpload;
        };

        $( hostAuthAuthType ).on( 'change', onHostAuthAuthTypeChanged );
        
        // Triggering the event handlers for the first time, for initialization purposes:
        onRestrictionTypeFieldChanged( null );
        onHostAuthAuthTypeChanged( null );
        var hmcScopeSet = null;
        // Loading the Scope Sets in the Scope Set Selector...
        $( getScopeSetsCmd ).on( 'executionSucceeded', $.proxy( function( event )
        {
          if( TV.typeIsObject( typeof event.response.payload.scopeSets ) )
          {
            /** @type {Array.<Object>} */ var scopeSets = event.response.payload.scopeSets;
            
            if( TV.typeIsObject( typeof scopeSets ) && TV.isInstanceOf( scopeSets, Array ) )
            {
              logger.info( 'Populating scope set selector (' + scopeSets.length + ' scope set(s))...' );
              $( scopeSetField ).empty();
              this.scopeSets = {};
              // Find HMC Dynamic Scope set
              $.each( scopeSets, function( scopeSetIndex, scopeSet )
              {
                if (pid !=null && pid ==scopeSet.pid) {
                    hmcScopeSet = scopeSet;
                    return false;
                }
              });
             // var containScopeSets = false;
              $.each( scopeSets, $.proxy( function( scopeSetIndex, scopeSet )
              {
                //To enable multiple credential remove length condition, might be needed for post 2.5
                if( TV.typeIsString( typeof scopeSet.displayName ) &&
                    TV.typeIsString( typeof scopeSet.pid         ) &&
                    scopeSet.displayName.startsWith(hmcScopeSet.displayName) &&
                    (scopeSet.credentials.length == 0 || isEditionMode) )
                {
                  //var containScopeSets = true;
                  /** @type {Element} */ var option = document.createElement( 'option' );

                  $( option ).text( scopeSet.displayName );
                  $( option ).prop( 'value', scopeSet.pid );
                  $( scopeSetField ).append( option );

                  this.scopeSets[ scopeSet.pid ] = scopeSet;
                }
                else
                {
                  logger.warn( 'Skipping invalid scope set (' + 'index'       + ': ' + scopeSetIndex        + '; '
                                                              + 'pid'         + ': ' + scopeSet.pid         + '; '
                                                              + 'displayName' + ': ' + scopeSet.displayName + ').' );
                }
              }, this ) );
             /* if(!containScopeSets && !isEditionMode){
                  alert("All HMC Dynamic Scope Set contains Credentials");
                  ( new ShowCommand( { action              : 'scopesetshmc/show'
                      , parameters : {pid : pid}} ) ).execute();
                  
              }*/

              if( isEditionMode && TV.typeIsObject( typeof credential.scopeSet ) && ( credential.scopeSet !== null ) )
              {
                $( scopeSetField ).val( credential.scopeSet.pid );
              }
            }
          }
        }
        , this ) );

        // Retrieving Scope Sets from server...
        getScopeSetsCmd.execute();

        // Return the form...
        return firstForm;
      };
    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.CredentialshmcHandler;
  }
);
