/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , ButtonFactory
         )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'DocumentationHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.DocumentationHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.DocumentationHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.DocumentationHandler.prototype ).constructor = tsa.cmd.handlers.DocumentationHandler;

    //-------------------------------------------------------------------------

    /**
     * Show main screen.
     */
    tsa.cmd.handlers.DocumentationHandler.prototype._index = function()
    {
    	 /** @type {Object}                    */ var i18n               = tsa.i18n;
    	 /** @type {tsa.widgets.ContentArea}   */ var contentArea        = null;
    	 /** @type {Element}                   */ var downloadDocButton  = null;
    	 /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer    = ButtonFactory.buildButtonContainer();

    	 // Initialize the content area (its title and its help document)...
    	 contentArea = new ContentArea ( { title   : i18n.labelDocumentationHeading
    	                                    , helpDoc : 'sshelp_documentation_panel.html' } );

    	 // Appending messages to the content area...
    	 contentArea.appendMessage( { text : i18n.labelDocumentationCaption } );
    	 contentArea.appendMessage( { text : i18n.labelDocConfigAssitDesc } );
    	 contentArea.appendMessage( { text : i18n.labelDocWhitePaperDesc } );
    	 contentArea.appendMessage( { text : i18n.labelDocSetupGuideDesc } );
    	 contentArea.appendMessage( { text : i18n.labelDocUpdateHistoryDesc } );

    	 // Appending button to content area...
   		 downloadDocButton = ButtonFactory.buildButton( { type : 'download' , grayout : false , label : i18n.buttonDownloadDocumentation ,
													      onclick : function( event ){tsa.cmd.handlers.DocumentationHandler.downloadDocumentation();} })

		 $( buttonContainer ).append( downloadDocButton );
    	 contentArea.append( buttonContainer );

    	 // Content area must be shown...
         this.show( contentArea.getDomNode() );

    }

    //-------------------------------------------------------------------------

    /**
     * Request for download of documentation.
     */
    tsa.cmd.handlers.DocumentationHandler.downloadDocumentation = function()
    {
      /** @type {tsa.cmd.Command} */ var cmd = null;

      cmd = new Command( { action  : 'documentation/download/TSADocumentation.zip'
                         , baseURL : Command.URL_SERVLET_CONTEXT
                         , blockUI : false
                         , ajax    : false } );
      cmd.execute();
    };

    return tsa.cmd.handlers.DocumentationHandler;
  }
);