/*
 * IBM Technical Support Appliance Version 2 Release 4 Modification 0
 *
 * (C) Copyright IBM Corp. 2018, 2019
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/data/Countries'
  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , TablePanel
          , FormPanel
          , ButtonFactory
          , Command
          , ShowCommand
          , Countries
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'RegistrationHandler' );
    
    //-------------------------------------------------------------------------

    /**
     * @constructor
     * @extends tsa.cmd.AbstractHandler
     * Parameters added by this class (in addition to super-class' parameters):
     *   none.
     *
     * @param {Object} parameters The parameters.
     */
    tsa.cmd.handlers.RegistrationHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.RegistrationHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.RegistrationHandler.prototype ).constructor = tsa.cmd.handlers.RegistrationHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.RegistrationHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n                   = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea            = new ContentArea ( { title   : i18n.labelRegistrationHeading
                                                                                             ,  helpDoc : 'sshelp_registration_panel.html' } );
      /** @type {tsa.widgets.FormPanel}     */ var formPanelContact       = new FormPanel( { title  : i18n.labelRegistrationContactHeading
                                                                                           , action : 'registration/ajaxUpdateRegistration' } );
      /** @type {tsa.widgets.FormPanel}     */ var formPanelLocation      = new FormPanel( { title  : i18n.labelRegistrationSyslocHeading} );
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer        = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getRegistrationCommand = null;
      /** @type {tsa.cmd.ShowCommand}       */ var showCmdCancel          = new ShowCommand( { action : 'main/index' } );
      /** @type {tsa.cmd.ShowCommand}       */ var showCmdSave            = new ShowCommand( { action : 'registration' , clearSystemMessages : false } );
      /** @type {Element}                   */ var selectCountries        = document.createElement( 'select' );
      
      contentArea.appendMessage( { text : i18n.labelRegistrationCaption } );
      
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactCo
                                       , required : true
                                       , hint     : i18n.labelRegistrationContactCoDesc
                                       , element  : $( '<input type="text" size="50" name="contactCompany" id="contactCompany" value="" required/>' ).get(0)
                                       } );
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactName
                                       , required : false
                                       , hint     : i18n.labelRegistrationContactNameDesc
                                       , element  : $( '<input type="text" size="50" name="contactName" id="contactName" value="" />' ).get(0)
                                       } );
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactPhone
                                       , required : false
                                       , hint     : i18n.labelRegistrationContactPhoneDesc
                                       , element  : $( '<input type="text" size="50" name="contactPhone" id="contactPhone" value="" />' ).get(0)
                                       } );
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactEmail
                                       , required : false
                                       , hint     : i18n.labelRegistrationContactEmailDesc 
                                       , element  : $( '<input type="text" size="50" name="contactEmail" id="contactEmail" value="" />' ).get(0)
                                       } );
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactIBMid
                                       , required : false
                                       , element  : $( '<input type="text" size="50" name="contactIBMid" id="contactIBMid" value="" />' ).get(0)
                                       } );
      
      /** @type {Element} */ var infoIBMIdDiv  = document.createElement( 'div' );
      /** @type {Element} */ var aRefIBMidLink1 = document.createElement("a");
      /** @type {Element} */ var aRefIBMidLink2 = document.createElement("a");
      
      $( aRefIBMidLink1 ).attr( 'href', i18n.labelRegistrationContactIBMidInstructionsLink1 );
      $( aRefIBMidLink1 ).attr( 'target', '_blank' );
      $( aRefIBMidLink1 ).append( i18n.labelRegistrationContactIBMidInstructionsLink1Text );
      
      $( aRefIBMidLink2 ).attr( 'href', i18n.labelRegistrationContactIBMidInstructionsLink2 );
      $( aRefIBMidLink2 ).attr( 'target', '_blank' );
      $( aRefIBMidLink2 ).append( i18n.labelRegistrationContactIBMidInstructionsLink2Text );
      
      $( infoIBMIdDiv ).css( { 'color'     : '#666'
                             , 'padding'   : '2px'
                             , 'font-size' : '0.9em' } );
      
      $( infoIBMIdDiv ).append( i18n.labelRegistrationContactIBMidInstructions1 );
      $( infoIBMIdDiv ).append( aRefIBMidLink1 );
      $( infoIBMIdDiv ).append( i18n.labelRegistrationContactIBMidInstructions2 );
      $( infoIBMIdDiv ).append( aRefIBMidLink2 );
      $( infoIBMIdDiv ).append( i18n.labelRegistrationContactIBMidInstructions3 );
      
      formPanelContact.appendFieldRow( { required : false
                                       , element  : $( infoIBMIdDiv ).get(0)
                                       } );
      
      formPanelContact.setDescription( i18n.labelRegistrationContactDesc );
      formPanelLocation.setDescription( i18n.labelRegistrationSyslocDesc );
      
      selectCountries.name = 'locationCountry';
      selectCountries.id   = 'locationCountry';
      
      $.each( Countries.getCountries( 'en' ), function( countryIndex, country )
      {
        /** @type {Element} */ var opt = document.createElement( 'option' );
        opt.value = country.id;
        $( opt ).text( country.name.en ); 
        $( selectCountries ).append( opt );
      } );
      
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocCountry
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocCountryDesc 
                                        , element  : selectCountries
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocState
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocStateDesc 
                                        , element  : $( '<input type="text" size="50" name="locationState" id="locationState" value="" required/>' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocZip
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocZipDesc 
                                        , element  : $( '<input type="text" size="50" name="locationZip" id="locationZip" value="" required/>' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocCity
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocCityDesc 
                                        , element  : $( '<input type="text" size="50" name="locationCity" id="locationCity" value="" required/>' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocAddress
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocAddressDesc 
                                        , element  : $( '<input type="text" size="50" name="locationAddress" id="locationAddress" value="" required/>' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocPhone
                                        , hint     : i18n.labelRegistrationSyslocPhoneDesc 
                                        , element  : $( '<input type="text" size="50" name="locationPhone" id="locationPhone" value="" />' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocBuilding
                                        , required : false
                                        , hint     : i18n.labelRegistrationSyslocBuildingDesc 
                                        , element  : $( '<input type="text" size="50" name="locationName" id="locationName" value=""/>' ).get(0)
                                        } );
      
      $( formPanelContact ).on( 'executionSucceeded', function( event )
      { 
        showCmdSave.execute();
      } );
      
      $( formPanelContact ).on( 'executionFailed', function( event )
      {
        if( TV.typeIsObject( typeof event.response.payload.errors ) )
        {
          for( /** @type {string} */ var fieldName in event.response.payload.errors )
          {
            formPanelContact.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
          }
        }
      } );
      
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonSave
          , onclick : function( event )
            {
              formPanelContact.submit();
            }
          }
        )
      );
      
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'cancel'
          , grayout : true
          , label   : i18n.buttonCancel
          , onclick : function( event )
            {
              showCmdCancel.execute();
            }
          }
        )
      );
      
      contentArea.append( formPanelContact.getDomNode() );
      contentArea.append( formPanelLocation.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
      
      //retrieving data from server
      getRegistrationCommand = new Command( {
          action:'registration/getRegistration'
        , clearSystemMessages:false
      } );
      
      // linking, master form submits slaves...
      formPanelContact.linkTo( formPanelLocation );
      
      $( getRegistrationCommand ).on( 'executionSucceeded', function(e) {
        for( var key in e.response.payload ) {
          $( '[name="' + key + '"]' ).val( e.response.payload[ key ] );
        }
      } );
      
      //cancel behavior
      getRegistrationCommand.execute(); 
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.RegistrationHandler;
  }
);