/*
 * IBM Technical Support Appliance Version 2 Release 5 Modification 0
 *
 * (C) Copyright IBM Corp. 2019
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/IconFactory'

  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , TablePanel
          , ButtonFactory
          , FormPanel
          , ShowCommand
          , Command
          , IconFactory)
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'ScopesetshmcHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.ScopesetshmcHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.ScopesetshmcHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ScopesetshmcHandler.prototype ).constructor = tsa.cmd.handlers.ScopesetshmcHandler;

    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.ScopesetshmcHandler.prototype._index = function()
    {
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( {  title         : i18n.labelDiscoHMCScopesHeading
                                                                                                  , helpDoc       : 'sshelp_discovery_hmc_credentials_view_panel.html'
                                                                                                  , showBackToTop : true } );
      /** @type {tsa.cmd.Command} */             var getScopesetsCmd         = new Command({     action        : 'scopesets/getScopeSets'
                                                                                              , parameters     : { 'filter' : 'HMC' }
                                                                                              , clearSystemMessages : false });

      /** @type {tsa.widgets.TablePanel} */      var tablePanel              = new TablePanel( {  title        : i18n.labelDiscoHMCScopesHeading
                                                                                                  , emptyMsg : i18n.labelDiscoHMCScopesNoSets
                                                                                                  , columns      : [ { id : 'name'   , label : i18n.labelDiscoHMCScopesName , width : '80%', sortable : false  }
                                                                                                                   , { id : 'action' , label : i18n.labelActions            , width : '20%' , sortable : false } ] } );

      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer            = ButtonFactory.buildButtonContainer();
      contentArea.appendMessage( { text : i18n.labelDiscoHMCScopesCaption } );


      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonDiscoHMCScopesAddSet
            , onclick : function( event )
              {
              (new ShowCommand( {
                   action : 'scopesetshmc/new'
                 , clearSystemMessages : false } )
               ).execute();
              }
            }
          )
        );

      getScopesetsCmd.execute();
      $(getScopesetsCmd).on('executionSucceeded', function(event){
        // Get the Scope Set and display
        /** @type {number} */ var scopesets = event.response.payload.scopeSets;
        /** @type {Array.<(string|Element)>} */ var result          = null;
        
        if( !TV.typeIsUndefined( typeof scopesets ) )
        {
            var hmcScopesets = [];
            $.each( scopesets, function( icaIdx, set ){
                //find HMC ScopeSet
                if(!set.name.endsWith("_LINUX") &&!set.name.endsWith("_AIX") &&!set.name.endsWith("_VIOS")) {
                    hmcScopesets.push(set);
                }
            });

          $.each( hmcScopesets, function( icaIdx, set ){
              /** @type {Element} */ var divName               = document.createElement("div");
              /** @type {Element} */ var aName                 = document.createElement("a");
              /** @type {Element} */ var hmcScopeSetName       = set.name;
              /** @type {Element} */ var hmcScopeSetNameLinux  = set.name+"_LINUX";
              /** @type {Element} */ var hmcScopeSetNameAix    = set.name+"_AIX";
              /** @type {Element} */ var hmcScopeSetNameVios   = set.name+"_VIOS";
              
              var clickDisplayHandler = function ( event ) {
                (new ShowCommand( {
                    action : event.data.URL
                  , clearSystemMessages : false
                  , parameters          : { pid : event.data.pid } } )
                 ).execute();
              }
              /**@type function() */
              var clickTriggerHandler = function ( event ) {
                tsa.clearSysMessages();
                var proceed = true;
                if(!TV.typeIsUndefined( typeof event.data.confirm )){
                  proceed = confirm(i18n.labelDeleteConfirmation + " '" + event.data.name + "' ?")
                }
                if(proceed){
                  var actionCmd = new Command( { action : event.data.URL
                                                      , clearSystemMessages : false  } );
                  actionCmd.execute();
                  $( actionCmd ).on( 'executionSucceeded', function( event )
                       {
                        (new ShowCommand( {
                           action : 'scopesetshmc/index'
                             , clearSystemMessages : false}
                          )).execute();
                        } );
                }
              };
              $( aName ).attr( 'href', '#' );
              $( aName ).on( 'click', { pid : set.pid , URL : 'scopesetshmc/show', pid : set.pid } , clickDisplayHandler);

              /** @type {Element} */ var dActions   = document.createElement("div");
              /** @type {Element} */ var editIcon   = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit   + " '" + set.name + "'"} );
              /** @type {Element} */ var runIcon    = IconFactory.buildActionIcon( { type : 'runb'     , caption : i18n.labelDiscoHMCStart   + " '"+set.name+"'"+ i18n.labelDiscoHMCDiscovery } );
              /** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete + " '" + set.name + "'"} );
              /** @type {Element} */ var statusIcon = null;
              if (set.enabled){
                statusIcon = IconFactory.buildActionIcon( { type : 'active'   , caption :  "'" + set.name +"'" + i18n.labelCollectionEnabled} );
              } else {
                statusIcon = IconFactory.buildActionIcon( { type : 'offline'   , caption : "'" + set.name +"'" + i18n.labelCollectionDisabled} );
              }
              $( editIcon   ).on( 'click', { URL : 'scopesetshmc/edit', pid : set.pid  } , clickDisplayHandler );
              $( statusIcon ).on( 'click', { URL : 'scopesets/ajaxToggle/'   + set.pid, name: set.name   } , clickTriggerHandler );
              $( runIcon    ).on( 'click', { URL : 'scopesetshmc/ajaxDiscover/' + set.pid, name: set.name } , clickTriggerHandler );
              $( deleteIcon ).on( 'click', { URL : 'scopesetshmc/ajaxDelete/'   + set.pid, name: set.name , confirm : true} , clickTriggerHandler );

              /** @type {Element} */ var expandIcon   = IconFactory.buildActionIcon( { type : 'expandb'   , caption : i18n.labelDiscoSchedExpandAll } );

              //$( divName    ).append( expandIcon);
              $( divName    ).append( tsa.NBSP);
              $(divName).append(aName);
              $( aName    ).append( set.name);


              /** @type {Element} */ var aRef    = document.createElement("a");
              $( aRef ).attr( 'href', '#' );
              $( aRef ).append( editIcon );
              $( dActions ).append( aRef );
              $( dActions ).append(tsa.NBSP);

              aRef    = document.createElement("a");
              $( aRef ).attr( 'href', '#' );
              $( aRef ).append( deleteIcon );
              $( dActions ).append( aRef );
              $( dActions ).append(tsa.NBSP);

              aRef    = document.createElement("a");
              $( aRef ).attr( 'href', '#' );
              $( aRef ).append( statusIcon );
              $( dActions ).append( aRef );
              $( dActions ).append(tsa.NBSP);

              aRef    = document.createElement("a");
              $( aRef ).attr( 'href', '#' );
              $( aRef ).append( runIcon );
              $( dActions ).append( aRef );
              $( dActions ).append(tsa.NBSP);

              result = [ divName , dActions ];

              tablePanel.appendRow( result );
          });

        }
       });
      //end logic to retrieve and display scopeset

      contentArea.append( tablePanel.getDomNode() );
      contentArea.append(buttonContainer);
      this.show( contentArea.getDomNode() );
    };
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesetshmcHandler.prototype._new = function(){
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( { title         : i18n.labelHMCScopeSetHeading
                                                                                , helpDoc       : 'sshelp_discovery_hmc_new_scope_set_panel.html'
                                                                                , showBackToTop : true } );
      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer         = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainerTestHMC  = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.FormPanel} */       var scopeSetPanel           = new FormPanel( { title  : i18n.labelHMCScopeSetEnterHeading } );
      /** @type {tsa.widgets.FormPanel}   */     var scopePanel              = new FormPanel( { title  : i18n.labelDiscoHMCScopesAddressHeading } );
      /** @type {tsa.widgets.FormPanel}   */     var hmcPanel                = new FormPanel( {           title   : i18n.labelDiscoHMCCredsEnterAccessHeading
                                                                                                  , description   : i18n.labelDiscoHMCCredsHostAuthEnterDesc });
      //startpanel for type of lpar
      /** @type {tsa.widgets.FormPanel}   */     var lparTypePanel           = new FormPanel( {           title   : i18n.labelDiscoHMCCredsEnterLPARSectionHeading
                                                                                                  , description   : i18n.labelDiscoHMCCredsEnterLPARSectionDesc });
     //endpanel for type of lpar
      /** @type {tsa.widgets.FormPanel}   */     var hmcAixPanel             = new FormPanel( {           title   : i18n.labelDiscoHMCCredsEnterAccessAixHeading
                                                                                                  , description   : i18n.labelDiscoHMCCredsHostAuthEnterAixDesc });
      /** @type {tsa.widgets.FormPanel}   */     var testFormAix             = new FormPanel( { title       : i18n.labelDiscoHMCCredsAixTestHeading
                                                                                              , description : i18n.labelDiscoHMCCredsTestDesc } );
      /** @type {tsa.widgets.FormPanel}   */     var hmcLinuxPanel           = new FormPanel( {           title   : i18n.labelDiscoHMCCredsEnterAccessLinuxHeading
                                                                                                  , description   : i18n.labelDiscoHMCCredsHostAuthEnterLinuxDesc });
      /** @type {tsa.widgets.FormPanel}   */     var testFormLinux           = new FormPanel( { title       : i18n.labelDiscoHMCCredsLinuxTestHeading
                                                                                              , description : i18n.labelDiscoHMCCredsTestDesc } );
      /** @type {tsa.widgets.FormPanel}   */     var hmcViosPanel            = new FormPanel( {           title   : i18n.labelDiscoHMCCredsEnterAccessViosHeading
                                                                                                  , description   : i18n.labelDiscoHMCCredsHostAuthEnterViosDesc });
      /** @type {tsa.widgets.FormPanel}   */     var testFormVios            = new FormPanel( { title       : i18n.labelDiscoHMCCredsViosTestHeading
                                                                                              , description : i18n.labelDiscoHMCCredsTestDesc } );
      //link all FormPanel
      scopeSetPanel.linkTo(scopePanel);
      scopePanel.linkTo(lparTypePanel);
      lparTypePanel.linkTo(hmcPanel);
      
      hmcPanel.linkTo(hmcViosPanel);
      hmcViosPanel.linkTo(testFormVios);
      testFormVios.linkTo(hmcAixPanel);
      hmcAixPanel.linkTo(testFormAix);
      testFormAix.linkTo(hmcLinuxPanel);
      hmcLinuxPanel.linkTo(testFormLinux);

      //create Scope sed fields
      scopeSetPanel.setDescription( i18n.labelHMCScopeSetEnterDesc);
      scopeSetPanel.appendFieldRow( { label    : i18n.labelHMCScopeSetEnterName
                                  , required : true
                                  , element  : $( '<input size="50" maxlength="250" name="name" id="name" value="" aria-required="true" class="ibm-text" type="text">' ).get(0)
                                  } );

      scopePanel.appendFieldRow( {  label    : i18n.labelDiscoHMCScopesAddressAddress
                                  , required : true
                                  , element  : $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0)
                                  , hint     : i18n.labelDiscoHMCScopesAddressAddressDesc } );
      
      var scopeSetGroup = $("<span class='ibm-input-group' role='group' aria-label='Scopes'></span>").get(0);
      
      //aix select
      var scopesetInput = $("<input name='lpars' class='checkBoxClass' type='checkbox' checked>").get(0);
      $(scopesetInput).on('click',function(){
          
          if(this.checked) {
              hmcAixPanel.show();
              testFormAix.show();
          } else {
              hmcAixPanel.hide();
              testFormAix.hide();
          }
      });
      $(scopesetInput).attr("id", "checkBoxAix");
      $(scopesetInput).attr("value", "AIX");
      var scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
      $(scopeLabel).text("AIX");
      $(scopeLabel).attr("for", "AIX");
      $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));
      //linux select
      scopesetInput = $("<input name='lpars' class='checkBoxClass' type='checkbox' checked>").get(0);
      $(scopesetInput).on('click',function(){
          console.log(this.value);
          
          if(this.checked) {
              hmcLinuxPanel.show();
              testFormLinux.show();
          } else {
              hmcLinuxPanel.hide();
              testFormLinux.hide();
          }
      });
      $(scopesetInput).attr("id", "checkBoxLinux");
      $(scopesetInput).attr("value", "LINUX");
      scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
      $(scopeLabel).text("LINUX");
      $(scopeLabel).attr("for", "LINUX");
      $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));
      
      //vios select
      scopesetInput = $("<input name='lpars' class='checkBoxClass' type='checkbox' checked>").get(0);
      $(scopesetInput).on('click',function(){
          console.log(this.value);
          
          if(this.checked) {
              hmcViosPanel.show();
              testFormVios.show();
          } else {
              hmcViosPanel.hide();
              testFormVios.hide();
          }
      });
      $(scopesetInput).attr("id", "checkBoxVios");
      $(scopesetInput).attr("value", "VIOS");
      scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
      $(scopeLabel).text("VIOS");
      $(scopeLabel).attr("for", "VIOS");
      $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));
      
      lparTypePanel.appendFieldRow( {  label    : i18n.labelDiscoHMCCredsEnterLPARSectionTypes
          , required : false
          , aRequire : false
          , element  : scopeSetGroup } );
      
      /*type function*/
      var i18nAuthType = function ( authtype ) {
          /** @type {string} */ var i18nLabel = '';

          if( !TV.typeIsString( typeof authtype ) )
          {
            authtype = '' + authtype;
          }

          switch( authtype )
          {
            case 'authType_default'    : i18nLabel = tsa.i18n.labelDiscoHMCCredsAuthTypeDefault   ; break;
            case 'authType_password'   : i18nLabel = tsa.i18n.labelDiscoHMCCredsAuthTypePassw     ; break;
            case 'authType_pki'        : i18nLabel = tsa.i18n.labelDiscoHMCCredsAuthTypePki       ; break;
            case 'authType_pki_scoped' : i18nLabel = tsa.i18n.labelDiscoHMCCredsAuthTypePkiScoped ; break;
            default                    : i18nLabel = authtype;
          }

          return i18nLabel;
      };
      // Create HMC Scope Set Credential fields, here we only allow all the types of credentials
      /** @type {Element} */ var hmcHostCredName      = $( '<input type="text"   id="HostAuth_name"  name="HostAuth_name"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var hmcHostUserName      = $( '<input type="text"   id="HostAuth_username"  name="HostAuth_username"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var hmcHostPassword1     = $( '<input type="password" id="HostAuth_password" name="HostAuth_password"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var hmcHostPassword2     = $( '<input type="password" id="HostAuth_password_confirm" name="HostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var hmcHostAuthType      = $( '<select name="HostAuth_authtype" id="HostAuth_authtype" >'
                                                          +   '<option value="authType_default" selected="selected"' + '>' + i18nAuthType( 'authType_default'    ) + '</option>'
                                                          +   '<option value="authType_password"'                    + '>' + i18nAuthType( 'authType_password'   ) + '</option>'
                                                          +   '<option value="authType_pki_scoped"'                  + '>' + i18nAuthType( 'authType_pki_scoped' ) + '</option>'
                                                          +   '<option value="authType_pki"'                         + '>' + i18nAuthType( 'authType_pki'        ) + '</option>'
                                                          + '</select>' ).get(0);
      /** @type {Element} */ var hmcHostUploadKey     = $( '<input type="file" id="HostAuth_sshKey" name="HostAuth_sshKey" />' ).get(0);
      /** @type {Element} */ var hmcHostUploadKeyType = document.createElement( 'p' );

      /** @type {Element} */ var hmcHostUploadKeyDiv  = document.createElement( 'div' );

      $( hmcHostUploadKeyType ).text( i18n.labelDiscoHMCCredsInternalSSHKey );

      $( hmcHostUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
      $( hmcHostUploadKeyDiv ).append( hmcHostUploadKeyType );
      $( hmcHostUploadKeyDiv ).append( hmcHostUploadKey     );

      /*type function()*/
      var onHostAuthAuthTypeChanged = function( event ){
          /** @type {Element} */ var panel         = event.data.hmcPanel;
          /** @type {Element} */ var uploadKey     = event.data.hmcHostUploadKey;
          /** @type {Element} */ var uploadKeyType = event.data.hmcHostUploadKeyType;
          /** @type {Element} */ var authType      = event.data.hmcHostAuthType;
          /** @type {Element} */ var hostPassword1 = event.data.hmcHostPassword1;
          /** @type {Element} */ var hostPassword2 = event.data.hmcHostPassword2;
          /** @type {Element} */ var row           = event.data.rowPanel;
          /** @type {string } */ var authSshKey    = event.data.hostAuthSshKey;
          
          /** @type {Element} */ var uploadKeyField      =  uploadKey ;
          /** @type {Element} */ var keyTypeMessage      =  uploadKeyType;

          /** @type {boolean} */ var pkiGlobal           = ( $( authType ).val() === 'authType_pki'        );
          /** @type {boolean} */ var pkiScoped           = ( $( authType ).val() === 'authType_pki_scoped' );
          /** @type {boolean} */ var isPKI               = (pkiGlobal || pkiScoped);
          /** @type {boolean} */ var hideSSHKeyRow       = !isPKI;
          /** @type {boolean} */ var hideSSHKeyUpload    = ( hideSSHKeyRow || pkiGlobal );
          /** @type {boolean} */ var disableSSHKeyUpload = ! pkiScoped;
          /** @type {boolean} */ var hideKeyTypeMessage  = pkiScoped ;

          /** @type {Element} */ var replaceButton       = document.createElement( 'button' );
          /** @type {boolean} */ var hideKeyTypeMessage  = pkiScoped ;

          $( row            )[ hideSSHKeyRow      ? 'hide' : 'show' ]();
          $( uploadKeyField )[ hideSSHKeyUpload   ? 'hide' : 'show' ]();
          $( keyTypeMessage )[ hideKeyTypeMessage ? 'hide' : 'show' ]();
          
          if (hideSSHKeyUpload){
              panel.removeValidation( authSshKey );
          }else{
              panel.addValidationRule( authSshKey, 'required' );
          }
          //remove required field for passphrase
          if (pkiScoped) {
              panel.removeValidation( $(hostPassword1).attr('id') );
              panel.removeValidation( $(hostPassword2).attr('id') );
          } else {
              panel.addValidationRule( $(hostPassword1).attr('id'), 'required' );
              panel.addValidationRule( $(hostPassword2).attr('id'), 'required' );
          }
          
          // Password or Passphare? use the right label...
          //$( hmcHostPassword1        ).closest( 'tr' ).find( 'label' ).text( isPKI ? i18n.labelDiscoHMCCredsSSlPassw1 : i18n.labelDiscoHMCCredsPassw1 );
          /** @type {string}  */ var labelValue= isPKI ? i18n.labelDiscoHMCCredsSSlPassw1 : i18n.labelDiscoHMCCredsPassw1 ;
          /** @type {Element} */ var labelNode = $( hostPassword1        ).closest( 'tr' ).find( 'label' );
          labelNode.empty();
          
          if (!isPKI) {
              labelNode.append(labelValue, $("<span class='ibm-required'> *</span>").get(0));
          } else {
              labelNode.append(labelValue);
          }
          labelValue = isPKI ? i18n.labelDiscoHMCCredsSSlPassw2 : i18n.labelDiscoHMCCredsPassw2 ;
          labelNode = $( hostPassword2        ).closest( 'tr' ).find( 'label' );
          labelNode.empty();
          if (!isPKI) {
              labelNode.append(labelValue, $("<span class='ibm-required'> *</span>").get(0));
          } else {
              labelNode.append(labelValue);
          }

          if( isPKI )
          {
            $( keyTypeMessage ).text( pkiGlobal ? i18n.labelDiscoHMCCredsInternalSSHKey
                                                : i18n.labelDiscoHMCCredsExternalSSHKey );
          }

      }

      hmcPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsNameParam     , required: true  , element: hmcHostCredName     } );
      hmcPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsUser     , required: true  , element: hmcHostUserName     } );
      hmcPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsPassw1   , required: true  , element: hmcHostPassword1    } );
      hmcPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsPassw2   , required: true  , element: hmcHostPassword2    } );
      hmcPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsAuthType , required: false , element: hmcHostAuthType     } );
      var rowPanel = hmcPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsSSHKey   , required: true  , element: hmcHostUploadKeyDiv } );
      hmcPanel.removeValidation( "HostAuth_sshKey" );
      $( hmcHostAuthType ).on( 'change',
              {
                 hmcPanel              : hmcPanel
               , hmcHostUploadKey      : hmcHostUploadKey
               , hmcHostUploadKeyType  : hmcHostUploadKeyType
               , hmcHostAuthType       : hmcHostAuthType
               , hmcHostPassword1      : hmcHostPassword1
               , hmcHostPassword2      : hmcHostPassword2
               , hmcHostUploadKeyDiv   : hmcHostUploadKeyDiv
               , rowPanel              : rowPanel
               , hostAuthSshKey        : "HostAuth_sshKey"
                }
             , onHostAuthAuthTypeChanged );
      
      //Default values to trigger change handler
      var HMCAuthValues  = new Object();
      HMCAuthValues.data = new Object();
      HMCAuthValues.data.hmcPanel              = hmcPanel;
      HMCAuthValues.data.hmcHostUploadKey      = hmcHostUploadKey;
      HMCAuthValues.data.hmcHostUploadKeyType  = hmcHostUploadKeyType;
      HMCAuthValues.data.hmcHostAuthType       = hmcHostAuthType;
      HMCAuthValues.data.hmcHostPassword1      = hmcHostPassword1;
      HMCAuthValues.data.hmcHostPassword2      = hmcHostPassword2;
      HMCAuthValues.data.hmcHostUploadKeyDiv   = hmcHostUploadKeyDiv;
      HMCAuthValues.data.rowPanel              = rowPanel;
      /** @type {Element} */ var testHMCButton  = ButtonFactory.buildButton( {   type    : 'arrow'
                                                                            , label   : i18n.buttonDiscoHMCCredsTest
                                                                            , onclick : $.proxy(function( event ) {
                                                                                          var crednameInput  = $("input[name='name']").val();
                                                                                          var usernameInput  = $("input[name='HostAuth_username']").val();
                                                                                          var passwordInput  = $("input[name='HostAuth_password']").val();
                                                                                          var authtypeInput  = $("select[name='HostAuth_authtype']").val();
                                                                                          var ipAddressInput =  $("input[name='aaddr']").val();
                                                                                          var elementFileID  = (authtypeInput.startsWith("authType_pki")) ? "#HostAuth_sshKey" : null;
                                                                                          this.testCredential(crednameInput, usernameInput, passwordInput, authtypeInput, elementFileID, ipAddressInput );
                                                                                        },this) } );
      var row = hmcPanel.layoutTableBody.insertRow(-1);
      var td = document.createElement("td");
      var pButton = document.createElement("p");
      $(pButton).append(testHMCButton);
      $(td).attr("colspan", 2);
      $(td).append(pButton);
      $(row).append(td);
      
      // Create HMC AIX Scope Set Credential fields, here we only allow password auth
      /** @type {Element} */ var aixHMCHostCredName      = $( '<input type="text"   id="HostAuthAix_name"  name="HostAuthAix_name"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var aixHMCHostUserName      = $( '<input type="text"   id="HostAuthAix_username"  name="HostAuthAix_username"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var aixHMCHostPassword1     = $( '<input type="password" id="HostAuthAix_password" name="HostAuthAix_password"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var aixHMCHostPassword2     = $( '<input type="password" id="HostAuthAix_password_confirm" name="HostAuthAix_password_confirm" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var aixHMCHostHostAuthType  = $( '<select name="HostAuthAix_authtype" id="HostAuthAix_authtype" >'
                                                            +   '<option value="authType_default" selected="selected"' + '>' + i18nAuthType( 'authType_default'    ) + '</option>'
                                                            +   '<option value="authType_password"'                    + '>' + i18nAuthType( 'authType_password'   ) + '</option>'
                                                            +   '<option value="authType_pki_scoped"'                  + '>' + i18nAuthType( 'authType_pki_scoped' ) + '</option>'
                                                            +   '<option value="authType_pki"'                         + '>' + i18nAuthType( 'authType_pki'        ) + '</option>'
                                                            + '</select>' ).get(0);
      /** @type {Element} */ var aixHMCHostUploadKey     = $( '<input type="file" id="HostAuthAix_sshKey" name="HostAuthAix_sshKey" />' ).get(0);
      /** @type {Element} */ var aixHMCHostUploadKeyType = document.createElement( 'p' );

      /** @type {Element} */ var aixHMCHostUploadKeyDiv  = document.createElement( 'div' );
      
      $( aixHMCHostUploadKeyType ).text( i18n.labelDiscoHMCCredsInternalSSHKey );

      $( aixHMCHostUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
      $( aixHMCHostUploadKeyDiv ).append( aixHMCHostUploadKeyType );
      $( aixHMCHostUploadKeyDiv ).append( aixHMCHostUploadKey     );
      
      hmcAixPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsNameParam     , required : true , element : aixHMCHostCredName     } );
      hmcAixPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsUser     , required : true , element : aixHMCHostUserName     } );
      hmcAixPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsPassw1   , required : true , element : aixHMCHostPassword1    } );
      hmcAixPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsPassw2   , required : true , element : aixHMCHostPassword2    } );
      hmcAixPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsAuthType , required : false , element : aixHMCHostHostAuthType     } );
      var rowAixPanel = hmcAixPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsSSHKey   , required: true  , element: aixHMCHostUploadKeyDiv } );
      hmcAixPanel.removeValidation( "HostAuthAix_sshKey" );
      $( aixHMCHostHostAuthType ).on( 'change',
              {
                 hmcPanel              : hmcAixPanel
               , hmcHostUploadKey      : aixHMCHostUploadKey
               , hmcHostUploadKeyType  : aixHMCHostUploadKeyType
               , hmcHostAuthType       : aixHMCHostHostAuthType
               , hmcHostPassword1      : aixHMCHostPassword1
               , hmcHostPassword2      : aixHMCHostPassword2
               , hmcHostUploadKeyDiv   : aixHMCHostUploadKeyDiv
               , rowPanel              : rowAixPanel
               , hostAuthSshKey        : "HostAuthAix_sshKey"
                }
             , onHostAuthAuthTypeChanged );
      //values to enable default configuration
      var AixAuthValues  = new Object();
      AixAuthValues.data = new Object();
      AixAuthValues.data.hmcPanel              = hmcAixPanel;
      AixAuthValues.data.hmcHostUploadKey      = aixHMCHostUploadKey;
      AixAuthValues.data.hmcHostUploadKeyType  = aixHMCHostUploadKeyType;
      AixAuthValues.data.hmcHostAuthType       = aixHMCHostHostAuthType;
      AixAuthValues.data.hmcHostPassword1      = aixHMCHostPassword1;
      AixAuthValues.data.hmcHostPassword2      = aixHMCHostPassword2;
      AixAuthValues.data.hmcHostUploadKeyDiv   = aixHMCHostUploadKeyDiv;
      AixAuthValues.data.rowPanel              = rowAixPanel;
      
      /** @type {Element} */ var testAIXButton  = ButtonFactory.buildButton( {   type    : 'arrow'
        , label   : i18n.buttonDiscoHMCCredsTest
        , onclick : $.proxy(function( event ) {
                      var crednameInput  = $("input[name='HostAuthAix_name']").val();
                      var usernameInput  = $("input[name='HostAuthAix_username']").val();
                      var passwordInput  = $("input[name='HostAuthAix_password']").val();
                      var ipAddressInput =  $("input[name='aaddraix']").val();
                      var authtypeInput  = $("select[name='HostAuthAix_authtype']").val();
                      var elementFileID  = (authtypeInput.startsWith("authType_pki")) ? "#HostAuthAix_sshKey" : null;
                      this.testCredential(crednameInput, usernameInput, passwordInput, authtypeInput, elementFileID, ipAddressInput );
                    },this) } );
      

      // Test form for Aix
      var testhostFieldAix = $("<input size='50' maxlength='250' name='aaddraix' id='aaddraix' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0);
      testFormAix.appendFieldRow( { label    : i18n.labelDiscoHMCScopesAddressAddress   
                                          , required : false 
                                          , element  : testhostFieldAix } );
      testFormAix.appendHR();
      testFormAix.appendJointRow( testAIXButton );
        
      /* 
       * Create HMC LINUX Scope Set Credential fields 
       * */
      /** @type {Element} */ var linuxHmcHostCredName      = $( '<input type="text"   id="HostAuthLinux_name"  name="HostAuthLinux_name"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var linuxHmcHostUserName      = $( '<input type="text"   id="HostAuthLinux_username"  name="HostAuthLinux_username"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var linuxHmcHostPassword1     = $( '<input type="password" id="HostAuthLinux_password" name="HostAuthLinux_password"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var linuxHmcHostPassword2     = $( '<input type="password" id="HostAuthLinux_password_confirm" name="HostAuthLinux_password_confirm" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var linuxHmcHostHostAuthType  = $( '<select name="HostAuthLinux_authtype" id="HostAuthLinux_authtype" >'
                                                          +   '<option value="authType_default" selected="selected"' + '>' + i18nAuthType( 'authType_default'    ) + '</option>'
                                                          +   '<option value="authType_password"'                    + '>' + i18nAuthType( 'authType_password'   ) + '</option>'
                                                          +   '<option value="authType_pki_scoped"'                  + '>' + i18nAuthType( 'authType_pki_scoped' ) + '</option>'
                                                          +   '<option value="authType_pki"'                         + '>' + i18nAuthType( 'authType_pki'        ) + '</option>'
                                                          +   '</select>' ).get(0);
      /** @type {Element} */ var linuxHmcHostUploadKey     = $( '<input type="file" id="HostAuthLinux_sshKey" name="HostAuthLinux_sshKey" />' ).get(0);
      /** @type {Element} */ var linuxHmcHostUploadKeyType = document.createElement( 'p' );
      
      /** @type {Element} */ var linuxHmcHostUploadKeyDiv  = document.createElement( 'div' );
      
      $( linuxHmcHostUploadKeyType ).text( i18n.labelDiscoHMCCredsInternalSSHKey );
      
      $( linuxHmcHostUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
      $( linuxHmcHostUploadKeyDiv ).append( linuxHmcHostUploadKeyType );
      $( linuxHmcHostUploadKeyDiv ).append( linuxHmcHostUploadKey     );
      
      hmcLinuxPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsNameParam     , required: true , element: linuxHmcHostCredName     } );
      hmcLinuxPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsUser     , required: true , element: linuxHmcHostUserName     } );
      hmcLinuxPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsPassw1   , required: true , element: linuxHmcHostPassword1    } );
      hmcLinuxPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsPassw2   , required: true , element: linuxHmcHostPassword2    } );
      hmcLinuxPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsAuthType , required : false , element : linuxHmcHostHostAuthType     } );
      var rowLinuxPanel = hmcLinuxPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsSSHKey   , required: true  , element: linuxHmcHostUploadKeyDiv } );
      hmcLinuxPanel.removeValidation( "HostAuthLinux_sshKey" );
      $( linuxHmcHostHostAuthType ).on( 'change',
              {
                 hmcPanel              : hmcLinuxPanel
               , hmcHostUploadKey      : linuxHmcHostUploadKey
               , hmcHostUploadKeyType  : linuxHmcHostUploadKeyType
               , hmcHostAuthType       : linuxHmcHostHostAuthType
               , hmcHostPassword1      : linuxHmcHostPassword1
               , hmcHostPassword2      : linuxHmcHostPassword2
               , hmcHostUploadKeyDiv   : linuxHmcHostUploadKeyDiv
               , rowPanel              : rowLinuxPanel
               , hostAuthSshKey        : "HostAuthLinux_sshKey"
                }
             , onHostAuthAuthTypeChanged );
      //values to enable default configuration
      var linuxAuthValues  = new Object();
      linuxAuthValues.data = new Object();
      linuxAuthValues.data.hmcPanel              = hmcLinuxPanel;
      linuxAuthValues.data.hmcHostUploadKey      = linuxHmcHostUploadKey;
      linuxAuthValues.data.hmcHostUploadKeyType  = linuxHmcHostUploadKeyType;
      linuxAuthValues.data.hmcHostAuthType       = linuxHmcHostHostAuthType;
      linuxAuthValues.data.hmcHostPassword1      = linuxHmcHostPassword1;
      linuxAuthValues.data.hmcHostPassword2      = linuxHmcHostPassword2;
      linuxAuthValues.data.hmcHostUploadKeyDiv   = linuxHmcHostUploadKeyDiv;
      linuxAuthValues.data.rowPanel              = rowLinuxPanel;
      
      

      /** @type {Element} */ var testLinuxButton  = ButtonFactory.buildButton( {    type    : 'arrow'
                                                                                  , label   : i18n.buttonDiscoHMCCredsTest
                                                                                  , onclick : $.proxy(function( event ) {
                                                                                                var crednameInput  = $("input[name='HostAuthLinux_name']").val();
                                                                                                var usernameInput  = $("input[name='HostAuthLinux_username']").val();
                                                                                                var passwordInput  = $("input[name='HostAuthLinux_password']").val();
                                                                                                var ipAddressInput =  $("input[name='aaddrlinux']").val();
                                                                                                var authtypeInput  = $("select[name='HostAuthLinux_authtype']").val();
                                                                                                var elementFileID  = (authtypeInput.startsWith("authType_pki")) ? "#HostAuthLinux_sshKey" : null;
                                                                                                this.testCredential(crednameInput, usernameInput, passwordInput, authtypeInput, elementFileID, ipAddressInput );
                                                                                              },this) } );
      // Test form for Linux
      var testhostFieldLinux = $("<input size='50' maxlength='250' name='aaddrlinux' id='aaddrlinux' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0)
      testFormLinux.appendFieldRow( { label    : i18n.labelDiscoHMCScopesAddressAddress   
                                          , required : false 
                                          , element  : testhostFieldLinux } );
      testFormLinux.appendHR();
      testFormLinux.appendJointRow( testLinuxButton );


      
      /*   
       * Create HMC VIOS Scope Set Credential fields
       * */
      /** @type {Element} */ var viosHmcHostName          = $( '<input type="text"     id="HostAuthVios_name"  name="HostAuthVios_name"  size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var viosHmcHostUserName      = $( '<input type="text"     id="HostAuthVios_username"  name="HostAuthVios_username"  size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var viosHmcHostPassword1     = $( '<input type="password" id="HostAuthVios_password" name="HostAuthVios_password"  size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var viosHmcHostPassword2     = $( '<input type="password" id="HostAuthVios_password_confirm" name="HostAuthVios_password_confirm" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var viosHmcHostHostAuthType  = $( '<select name="HostAuthVios_authtype" id="HostAuthVios_authtype" >'
                                                                +   '<option value="authType_default" selected="selected"' + '>' + i18nAuthType( 'authType_default'    ) + '</option>'
                                                                +   '<option value="authType_password"'                    + '>' + i18nAuthType( 'authType_password'   ) + '</option>'
                                                                +   '<option value="authType_pki_scoped"'                  + '>' + i18nAuthType( 'authType_pki_scoped' ) + '</option>'
                                                                +   '<option value="authType_pki"'                         + '>' + i18nAuthType( 'authType_pki'        ) + '</option>'
                                                                +   '</select>' ).get(0);
      /** @type {Element} */ var viosHmcHostUploadKey     = $( '<input type="file" id="HostAuthVios_sshKey" name="HostAuthVios_sshKey" />' ).get(0);
      /** @type {Element} */ var viosHmcHostUploadKeyType = document.createElement( 'p' );
      
      /** @type {Element} */ var viosHmcHostUploadKeyDiv  = document.createElement( 'div' );
      
      $( viosHmcHostUploadKeyType ).text( i18n.labelDiscoHMCCredsInternalSSHKey );
      
      $( viosHmcHostUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
      $( viosHmcHostUploadKeyDiv ).append( viosHmcHostUploadKeyType );
      $( viosHmcHostUploadKeyDiv ).append( viosHmcHostUploadKey     );

      hmcViosPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsNameParam , required : true , element : viosHmcHostName         } );
      hmcViosPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsUser      , required : true , element : viosHmcHostUserName     } );
      hmcViosPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsPassw1    , required : true , element : viosHmcHostPassword1    } );
      hmcViosPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsPassw2    , required : true , element : viosHmcHostPassword2    } );
      
      hmcViosPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsAuthType , required : false , element : viosHmcHostHostAuthType     } );
      var rowViosPanel = hmcViosPanel.appendFieldRow( { label: i18n.labelDiscoHMCCredsSSHKey   , required: true  , element: viosHmcHostUploadKeyDiv } );
      hmcViosPanel.removeValidation( "HostAuthVios_sshKey" );
      $( viosHmcHostHostAuthType ).on( 'change',
              {  hmcPanel              : hmcViosPanel
               , hmcHostUploadKey      : viosHmcHostUploadKey
               , hmcHostUploadKeyType  : viosHmcHostUploadKeyType
               , hmcHostAuthType       : viosHmcHostHostAuthType
               , hmcHostPassword1      : viosHmcHostPassword1
               , hmcHostPassword2      : viosHmcHostPassword2
               , hmcHostUploadKeyDiv   : viosHmcHostUploadKeyDiv
               , rowPanel              : rowViosPanel          
               , hostAuthSshKey        : "HostAuthVios_sshKey"}, onHostAuthAuthTypeChanged );
      //Default values to trigger change handler
      var viosAuthValues  = new Object();
      viosAuthValues.data = new Object();
      viosAuthValues.data.hmcPanel              = hmcViosPanel;
      viosAuthValues.data.hmcHostUploadKey      = viosHmcHostUploadKey;
      viosAuthValues.data.hmcHostUploadKeyType  = viosHmcHostUploadKeyType;
      viosAuthValues.data.hmcHostAuthType       = viosHmcHostHostAuthType;
      viosAuthValues.data.hmcHostPassword1      = viosHmcHostPassword1;
      viosAuthValues.data.hmcHostPassword2      = viosHmcHostPassword2;
      viosAuthValues.data.hmcHostUploadKeyDiv   = viosHmcHostUploadKeyDiv;
      viosAuthValues.data.rowPanel              = rowViosPanel;
      

      /** @type {Element} */ var testVIOSButton  = ButtonFactory.buildButton( {     type    : 'arrow'
                                                                                  , label   : i18n.buttonDiscoHMCCredsTest
                                                                                  , onclick : $.proxy(function( event ) {
                                                                                                var crednameInput  = $("input[name='HostAuthVios_name']").val();
                                                                                                var usernameInput  = $("input[name='HostAuthVios_username']").val();
                                                                                                var passwordInput  = $("input[name='HostAuthVios_password']").val();
                                                                                                var ipAddressInput =  $("input[name='aaddrvios']").val();
                                                                                                var authtypeInput  = $("select[name='HostAuthVios_authtype']").val();
                                                                                                var elementFileID  = (authtypeInput.startsWith("authType_pki")) ? "#HostAuthVios_sshKey" : null;
                                                                                                this.testCredential(crednameInput, usernameInput, passwordInput, authtypeInput, elementFileID, ipAddressInput );
                                                                                               },this) } );
      
      var testhostFieldVios = $("<input size='50' maxlength='250' name='aaddrvios' id='aaddrvios' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0)
      testFormVios.appendFieldRow( { label    : i18n.labelDiscoHMCScopesAddressAddress   
                                          , required : false 
                                          , element  : testhostFieldVios } );
      testFormVios.appendHR();
      testFormVios.appendJointRow( testVIOSButton );

      
      
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonSave
            , onclick : function( event )
              {
              scopeSetPanel.setActionAttribute("scopesetshmc/ajaxCreate/");


              scopeSetPanel.submit();
              $( scopeSetPanel ).on( 'executionSucceeded', function( event )
              {
                (new ShowCommand( {
                  action : 'scopesetshmc/index'
                    , clearSystemMessages : false
                    , parameters : { pid : event.response.payload.pid} } )
                ).execute();

              } );
              $( scopeSetPanel ).on( 'executionFailed', function( event )
                  {
                    if( TV.typeIsObject( typeof event.response.payload.errors ) )
                    {
                      for( /** @type {string} */ var fieldName in event.response.payload.errors )
                      {
                          scopeSetPanel.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                      }
                      delete event.response.payload.errors;
                    }

                  } );
              }
            }
          )
        );
      
      // call onHostAuthAuthTypeChanged to trigger default values.
      onHostAuthAuthTypeChanged( HMCAuthValues );
      onHostAuthAuthTypeChanged( AixAuthValues );
      onHostAuthAuthTypeChanged( linuxAuthValues );
      onHostAuthAuthTypeChanged( viosAuthValues );
      
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );

      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'cancel'
            , grayout : true
            , label   : i18n.buttonCancel
            , onclick : function( event )
              {
              (new ShowCommand( {
                action : 'scopesetshmc/index'
                  , clearSystemMessages : false}
               )).execute();
              }
            }
          )
        );
      contentArea.append( scopeSetPanel.getDomNode() );
      contentArea.append( scopePanel.getDomNode() );
      contentArea.append( lparTypePanel.getDomNode() );
      contentArea.append( hmcPanel.getDomNode() );
      contentArea.append( hmcViosPanel.getDomNode() );
      contentArea.append( hmcAixPanel.getDomNode() );
      contentArea.append( hmcLinuxPanel.getDomNode() );
      


      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
    };
    

    //-------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesetshmcHandler.prototype._show = function(){
      // Redirect to edit
      this._edit();
    };
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesetshmcHandler.prototype._edit = function() {
      /** @type {string}                  */     var pid                     = this.getParameter('pid');
      /** @type {string}                  */     var name                    = null;
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( { title         : i18n.labelHMCScopeSetHeading
                                                                                , helpDoc       : 'sshelp_discovery_hmc_show_scopes_set_panel.html' } );
      /** @type {tsa.cmd.Command} */             var getScopesetCmd          = new Command({ action:'scopesets/getScopeSet'
                                                                                            ,clearSystemMessages:false
                                                                                            ,  parameters          : { name : pid } });
      
      /** @type {tsa.widgets.TablePanel} */      var tablePanel              = new TablePanel( {  title        : i18n.labelHMCScopeSetGeneralHeading
                                                                                                , columns      : [ { id : 'name'   , width : '80'}
                                                                                                                 , { id : 'value' } ] } );

      /** @type {tsa.widgets.TablePanel} */      var tablePanelCred          = new TablePanel( {  title        : i18n.labelDiscoHMCCredsCredentialsHeading
                                                                                                  , emptyMsg   : i18n.labelDiscoHMCCredsNoCreds
                                                                                                  , columns    : [  { id : 'name'          , label : i18n.labelDiscoHMCCredsName                , width : '20%' , sortable : false }
                                                                                                                  , { id : 'type'          , label : i18n.labelDiscoCredsType                , width : '20%' , sortable : false }
                                                                                                                  , { id : 'authType'      , label : i18n.labelDiscoCredsAuthenticationType  , width : '20%' , sortable : false }
                                                                                                                  , { id : 'userName'      , label : i18n.labelDiscoHMCCredsUserName            , width : '15%' , sortable : false }
                                                                                                                  //, { id : 'lastPwdChange' , label : i18n.labelDiscoCredsPasswordChangedDate , width : '15%' , sortable : false }
                                                                                                                  , { id : 'restrictions'  , label : i18n.labelDiscoHMCCredsRestrictions        , width : '15%' , sortable : false }
                                                                                                                  , { id : 'actions'       , label : i18n.labelActions                       , width : '15%' , sortable : false } ] } );
      /** @type {tsa.widgets.TablePanel} */      var tablePanel2              = new TablePanel( {  title       : i18n.SummaryForScopesTable
                                                                                                 , emptyMsg     : i18n.labelScopeSetNoElements
                                                                                                 , columns      : [  { id : 'type'     , label : i18n.labelHMCScopeSetType  , width : '20%'}
                                                                                                                   , { id : 'value'    , label : i18n.labelHMCScopeSetValue , width : '30%'}
                                                                                                                   , { id : 'desc'     , label : i18n.labelHMCScopeSetDesc  , width : '40%'}
                                                                                                                   , { id : 'actions'  , label : i18n.labelActions          , width : '15%'}
                                                                                                                   ] });
      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer        =  ButtonFactory.buildButtonContainer();

      //TODO create the label
      /** @type {element} */                     var buttonNewCred          =  ButtonFactory.buildButton({  type    : 'arrow'
                                                                                                          , grayout : true
                                                                                                          , label   : i18n.buttonDiscoHMCCredsAdd
                                                                                                          , onclick : function( event )
                                                                                                            {
                                                                                                              (new ShowCommand( {
                                                                                                                action : 'credentialshmc/new'
                                                                                                              , clearSystemMessages : false
                                                                                                              , parameters :{ pid : pid }} )
                                                                                                              ).execute();
                                                                                                            }
                                                                                                          });
      var infoParagraph = document.createElement("div");
      
      getScopesetCmd.execute();
      $(getScopesetCmd).on('executionSucceeded', function(event){
        tablePanel.appendRow( [i18n.labelHMCScopeSetName,event.response.payload.scopeSet.displayName.replace("_HMCCrt_","")] );
        name = event.response.payload.scopeSet.displayName;
        if ( !TV.typeIsUndefined( typeof event.response.payload.scopeSet.scopeElements ) )
        {
          $.each( event.response.payload.scopeSet.scopeElements, function( idx, element ){
            /** @type {string} */ var type = null;
             if(element.type==0){
               type = i18n.labelScopeSetAddress;
             }
             /**@type function() */
             var clickTriggerHandler = function ( event ) {
               tsa.clearSysMessages();
               var proceed = true;
               if(!TV.typeIsUndefined( typeof event.data.confirm )){
                 proceed = confirm(i18n.labelDeleteConfirmation + " '" + event.data.name + "' ?")
               }
               if(proceed){
                 var actionCmd = new Command( { action : event.data.URL
                                                     , clearSystemMessages : false });
                 actionCmd.execute();
                 $( actionCmd ).on( 'executionSucceeded', function( event )
                      {
                       (new ShowCommand( {
                          action : 'scopesetshmc/edit'
                          , clearSystemMessages : false
                          , parameters : { pid : pid }}
                         )).execute();
                       } );
               }
             };
             /** @type {Element} */ var dActions   = document.createElement("div");
             /** @type {Element} */ var editIcon   = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit   + " '" + element.externalForm + "'"} );
             /** @type {Element} */ var runIcon    = IconFactory.buildActionIcon( { type : 'runb'     , caption : i18n.labelDiscoStart   + " '" + element.externalForm+"'"+ i18n.labelDiscoDiscovery } );
             ///** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete + " '" + element.externalForm + "'"} );
             $( editIcon   ).on( 'click',  function(){
               (new ShowCommand( {
                 action : 'scopeshmc/edit'
                   , clearSystemMessages : false
                   , parameters : {  pid    : element.pid
                                   ,  sepid : pid }
                                 })).execute();
             } );
             $( runIcon    ).on( 'click', { URL : "scopesetshmc/ajaxDiscover/" + pid } , clickTriggerHandler);
             //$( deleteIcon ).on( 'click', { URL : "scopes/ajaxDelete/" + element.pid , name : element.externalForm  , confirm : true } , clickTriggerHandler);
             /** @type {Element} */ var aRef    = document.createElement("a");
             $( aRef ).attr( 'href', '#' );
             $( aRef ).append( editIcon );
             $( dActions ).append( aRef );
             $( dActions ).append(tsa.NBSP);
             
             aRef    = document.createElement("a");
             $( aRef ).attr( 'href', '#' );
             $( aRef ).append( runIcon );
             $( dActions ).append( aRef );
             $( dActions ).append(tsa.NBSP);
             var displayName = element.displayName.indexOf("_HMCCRT")>-1 ? element.externalForm : element.displayName;

             tablePanel2.appendRow( [type, element.externalForm, displayName, dActions ] );
            });
              
          /** @type {Element} */ var selectIPHMC          = document.createElement( 'select' );
          // Timezone selector properties...
          selectIPHMC.name = "timezone";
          selectIPHMC.id   = "timezone";

          /** @type {Element} */ var opt = document.createElement( 'option' );
          opt.value = "Unassigned";

          $( opt ).text( "Unassigned" );
          $( opt ).attr('selected' , 'selected');
          $( selectIPHMC ).append( opt );

          $.each( event.response.payload.scopeSet.scopeElements, function( idx, element ){
              opt = document.createElement( 'option' );
              opt.value = element.pid;
              $( opt ).text( element.externalForm );
              $( selectIPHMC ).append( opt );

          });
          } else {
            //hide if there is no scopes to move
            $(buttonNewCred).hide();
          }
        //create sidebar
        /** @type {element} */ var paragraph       = document.createElement("p");
        $(infoParagraph).append(paragraph);
        var hmcName           = name.replace("_HMCCrt_","")
        var scopeSetNameLinux = hmcName+"_LINUX";
        var scopeSetNameVIOS  = hmcName+"_VIOS";
        var scopeSetNameAix   = hmcName+"_AIX";
        var templateInfo = function(value) {
            /** @type {Element}*/ var pTemplate  = document.createElement("p");
            /** @type {Element}*/ var brTemplate = $("<br class='break'>").get(0);
            $(pTemplate).append(value, brTemplate);
            return pTemplate;
        }
        var getPartitionScopeSet = new Command( {     action           : 'scopesets/getScopeSets'
                              , parameters           : { 'filter' : 'HMC'}
                              , clearSystemMessages  : false } );
        getPartitionScopeSet.execute();
        $( getPartitionScopeSet ).on( 'executionSucceeded', function( event )
        {
          /** @type {number} */ var scopesets = event.response.payload.scopeSets;
          
          if( !TV.typeIsUndefined( typeof scopesets ) )
          {
            var missingPartitionCreation = true;
            $.each( scopesets, function( icaIdx, set ) {
              if(set.name.startsWith(hmcName) && set.name.match(/(_AIX|_VIOS|_LINUX)$/) && set.enabled){
                $(infoParagraph).append(templateInfo(set.name));
                missingPartitionCreation = false;
              }
            });
            if(missingPartitionCreation){
              $(infoParagraph).append(templateInfo(i18n.labelHMCNonePartition));
            }
          }
        });
        
        
      });
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );

      $( buttonContainer ).append(buttonNewCred);

      //create credential panel
      //start logic to retrieve and display credentials
      var getCredCommand = new Command( {     action           : 'credentials/getCredentials'
                                        , parameters           : { 'filter' : 'EVERYTHING'}
                                        , clearSystemMessages  : false } );
      getCredCommand.execute();
      $( getCredCommand ).on( 'executionSucceeded', $.proxy( function( event )
      {
          //here we use the new fucnction
          /** @type {Array.<Object>}         */ var credentials     = event.response.payload.credentials;

          $.proxy(this.retrieveHMCCredentials(tablePanelCred, buttonContainer, credentials,pid,name),this);
      }
      , this ) );

    
      //start side bar
      contentArea.appendSidebarPanel( { title: i18n.labelHMCScopeSetPartitionHeading, body: infoParagraph } );
      
      contentArea.append(tablePanel.getDomNode());
      contentArea.append(tablePanel2.getDomNode());
      contentArea.append(tablePanelCred.getDomNode());
      this.show( contentArea.getDomNode() );
    };
    /**
     * Delete credential - event handler.
     * @param {string} credentialPID The credential PID.
     */
    tsa.cmd.handlers.ScopesetshmcHandler.prototype.deleteCredentialClicked = function( credentialName, credentialPID , pid)
    {
      if( TV.typeIsString( typeof credentialPID ) && ( credentialPID.length > 0 ) )
      {
        if( window.confirm( tsa.i18n.labelDeleteConfirmation + ' "' + credentialName + '"?' ) )
        {
          /** @type {tsa.cmd.Command} */ var deleteCmd = new Command( { action     : 'credentials/deleteCredential'
                                                                      , parameters : { 'pid' : credentialPID } } );

          $( deleteCmd ).on( 'executionSucceeded', function( event )
          {
            if ( !TV.typeIsUndefined( typeof event.response.sysMessages[0] ) )
            {
            //clear deafult message and remove scope name the extension
             tsa.clearSysMessages()
             tsa.printSysMessage( { level   : 'info'
                 , message : event.response.sysMessages[0].message.replace("_HMCCrt_","") } );
            }
            ( new ShowCommand( { action              : 'scopesetshmc/show'
                               , parameters : { pid : pid }
                               , clearSystemMessages : false } ) ).execute();
          } );

          deleteCmd.execute();
        }
      }
      else
      {
        logger.warn( 'Invalid credential PID. Deletion request did not proceed.' );
      }
    };
    tsa.cmd.handlers.ScopesetshmcHandler.prototype.retrieveHMCCredentials = function( tablePanelCred , buttonContainer, credentials, pid, scopesetName) {

        /** @type {Object}                 */ var i18n            = tsa.i18n;
        /** @type {tsa.widgets.TablePanel} */ var tablePanel      = null;
        /** @type {Object}                 */ var tablePanelProps = { title : i18n.labelDiscoCredsCredentialsHeading };
        var scopeSetNameLinux = scopesetName.replace("_HMCCrt_","_LINUX_HMCCrt_");
        var scopeSetNameVIOS  = scopesetName.replace("_HMCCrt_","_VIOS_HMCCrt_");
        var scopeSetNameAix   = scopesetName.replace("_HMCCrt_","_AIX_HMCCrt_");

        if( TV.typeIsObject( typeof credentials ) && TV.isInstanceOf( credentials, Array ) && ( credentials.length > 0 ) )
        {
          /** @type {number}         */ var scopeSetsMissing = 0;
          /** @type {Array.<Object>} */ var columnData       = [];
          var prunedCred = [];
          var isScopeSetMissingCred = false;
          var missingHMCCred        = true;
          var missingHMCLinuxCred   = true;
          var missingHMCAixCred     = true;
          var missingHMCViosCred    = true;
          // Prune credential not assigned to current hmc scopeset
          $.each( credentials, function( credIndex, cred )
          {
            scopeSetsMissing += ( cred.scopeSetMissing ? 1 : 0 );
            if(cred.name.endsWith("_HMCCrt_") && (cred.scopeSetName===scopesetName ||cred.scopeSetName===scopeSetNameLinux ||cred.scopeSetName===scopeSetNameVIOS ||cred.scopeSetName===scopeSetNameAix )){
                prunedCred.push(cred);
            }
          } );
          // This box is to identify if a credential is missing
          $.each( prunedCred, function( credIndex, cred )
          {
            if(cred.scopeSetName===scopesetName){
                missingHMCCred = false;
                return true;
            } else if (cred.scopeSetName===scopeSetNameLinux){
                missingHMCLinuxCred = false;
                return true;
            } else if (cred.scopeSetName!==scopeSetNameVIOS ){
                missingHMCViosCred = false;
                return true;
            } else if (cred.scopeSetName!==scopeSetNameAix ){
                missingHMCAixCred = false;
                return true;
            }
          } );
          if(missingHMCCred||missingHMCLinuxCred||missingHMCViosCred||missingHMCAixCred){
              isScopeSetMissingCred = true;
          }
          credentials=prunedCred;

          columnData.push( { id : 'name'          , label : i18n.labelDiscoCredsName                , width : '20%' , sortable : false }
                         , { id : 'type'          , label : i18n.labelDiscoCredsType                , width : '20%' , sortable : false }
                         , { id : 'userName'      , label : i18n.labelDiscoCredsUserName            , width : '15%' , sortable : false }
                         , { id : 'lastPwdChange' , label : i18n.labelDiscoCredsPasswordChangedDate , width : '15%' , sortable : false }
                         , { id : 'restrictions'  , label : i18n.labelDiscoCredsRestrictions        , width : '15%' , sortable : false }
                         , { id : 'actions'       , label : i18n.labelActions                       , width : '15%' , sortable : false } );

          if( scopeSetsMissing > 0 )
          {
            columnData[ columnData.length - 1 ].width = '10%';

            columnData.push( { id       : 'scopeSetMissing'
                             , label    : ''
                             , width    : '5%'
                             , sortable : false } );
          }

          tablePanelProps.columns = columnData;
        }
        else
        {
          tablePanelProps.emptyMsg = i18n.labelDiscoCredsNoCreds;
        }

        tablePanel = new TablePanel( tablePanelProps );

        $.each( credentials, function( credIndex, cred )
        {

           //if scopeset is null or does not have the tag in the name then continue
           if(!cred.name.endsWith("_HMCCrt_") ){
               return;
           }
           cred.name = cred.name.substring(0,cred.name.indexOf("_HMCCrt_"))

          /** @type {Array.<Element>} */ var elements     = [];
          /** @type {Element}         */ var nameSpan     = document.createElement( 'span' );
          /** @type {Element}         */ var nameAnchor   = document.createElement( 'a' );
          /** @type {Element}         */ var scopeSetSpan = document.createElement( 'span' );
          /** @type {Element}         */ var actionsSpan  = document.createElement( 'span' );
          /** @type {Element}         */ var editIcon     = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit     + ' "' + cred.name + '"' } );
          /** @type {Element}         */ var deleteIcon   = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete   + ' "' + cred.name + '"' } );

          /** @type {boolean}         */ var isFirst      = ( credIndex === 0 );
          /** @type {boolean}         */ var isLast       = ( credIndex === ( credentials.length - 1 ) );

          // Column 1 - Error mark:
          if( scopeSetsMissing > 0 )
          {
            elements.push( IconFactory.buildSeverityIcon( { severity : 'ERROR' } ) );
          }
          // Column 2 - Name:
          nameAnchor.href = '#';
          $( nameAnchor ).addClass( 'ibm-feature-link' );
          $( nameAnchor ).css( { 'font-weight' : 'bold' } );
          $( nameAnchor ).text( cred.name );
          $( nameAnchor ).on( 'click', function( event )
          {
              ( new ShowCommand( { 
                          action : 'credentialshmc/show/' + cred.pid 
                          , parameters          : { pid : pid }
                          } ) ).execute();

              //tsa.cmd.handlers.CredentialsHandler.showCredentialClicked( cred.pid );

            try
            {
              event.stopPropagation();
            } catch ( error ) { /* Ignore */ }

            return false;
          } );

          $( nameSpan ).append( nameAnchor );
          $( nameSpan ).append( document.createTextNode( tsa.isDebugModeOn() ? ' (' + cred.properties.order + ')' : '' ) );
          elements.push( nameSpan );

          // Column 3 - Class label:
          elements.push( cred.authClassLabel );
          /*Function*/
          var i18nAuthType = function ( authtype )
          {
            /** @type {string} */ var i18nLabel = '';
            
            if( !TV.typeIsString( typeof authtype ) )
            {
              authtype = '' + authtype;
            }
            
            switch( authtype )
            {
              case 'authType_default'    : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypeDefault   ; break;
              case 'authType_password'   : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypePassw     ; break;
              case 'authType_pki'        : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypePki       ; break;
              case 'authType_pki_scoped' : i18nLabel = tsa.i18n.labelDiscoCredsAuthTypePkiScoped ; break;
              default                    : i18nLabel = authtype;
            }
            
            return i18nLabel;
          }
          elements.push( i18nAuthType(cred.properties.authtype));
          // Column 4 - User name:
          elements.push( ( cred.userName !== null ) ? cred.userName : '' );

          // Column 5 - Password last changed:
          //elements.push( ( cred.passwordLastChanged !== null ) ? cred.passwordLastChanged : i18n.labelDiscoCredsPasswordChangeNotAvailable );

          // Column 6 - Scope Set:
          $( scopeSetSpan ).text( ( cred.scopeSetName !== null ) ? cred.scopeSetName.replace("_HMCCrt_","") : '' );
          $( scopeSetSpan ).css( { 'color' : ( cred.scopeSetMissing ? 'red' : 'black' ) } );
          elements.push( scopeSetSpan );

          // Column 7 - Actions:
          $( actionsSpan ).append( editIcon   );
          $( actionsSpan ).append( deleteIcon   );

          $.each( [ editIcon, deleteIcon ], function( iconIndex, icon )
          {
            $( icon ).css( { 'cursor'       : 'pointer'
                           , 'margin-right' : '0.5em' } );
          } );

          $( editIcon     ).on( 'click', function( event ) { tsa.cmd.handlers.ScopesetshmcHandler.prototype.editCredentialClicked( cred.pid, pid );              } );
          $( deleteIcon   ).on( 'click', function( event ) { tsa.cmd.handlers.ScopesetshmcHandler.prototype.deleteCredentialClicked( cred.name, cred.pid , pid); } );

          elements.push( actionsSpan );

          // Append the row...
          tablePanelCred.appendRow( elements );

        } );
        if(isScopeSetMissingCred){
          var rowButton    = tablePanelCred.tableBody.insertRow(-1);
          var tdButton     = document.createElement("td");
          var tdbuttonFill = document.createElement("td");
          $(tdButton).append( buttonContainer );
          $(tdButton).attr("colspan", 2);
          $(tdbuttonFill).attr("colspan", 4);
          $(rowButton).append(tdButton,tdbuttonFill);
        }
        
    };
    //-------------------------------------------------------------------------
    /**
     * Edit credential - event handler.
     * @param {string} credentialPID The credential PID.
     */

    tsa.cmd.handlers.ScopesetshmcHandler.prototype.editCredentialClicked = function( credentialPID , pid )
    {
      ( new ShowCommand( { action : 'credentialshmc/edit/' + credentialPID
          , parameters : { pid : pid } } ) ).execute();
    };
    /**
     * Move credential - event handler.
     * @param {string} credentialPID The credential PID.
     * @param {number} offset        Negative to move up, positive to move down.
     */
    tsa.cmd.handlers.ScopesetshmcHandler.prototype.testCredential = function( credName, username, password, authType, elementFileID, ipadd )
    {   
        tsa.clearSysMessages();
        /** @type {tsa.widgets.FormPanel} */       var testCred  = new FormPanel( {     action            : 'credentials/ajaxTestCredential'
                                                                                    , printSystemMessages : false} );
        var sshFileElement = null;
        if (elementFileID != null)
        {
          sshFileElement = $(elementFileID).get(0);
        }
        
        var restoreID = function() 
        {
          if (elementFileID != null) 
          {
            sshFileElement.name = elementFileID.substring(1);
            sshFileElement.id   = elementFileID.substring(1);
          }
        }
        
        
        testCred.putHiddenInput("CredentialType","HostAuth");
        testCred.putHiddenInput("name","JustForTestValueDummy01");
        testCred.putHiddenInput("HostAuth_username",username);
        testCred.putHiddenInput("HostAuth_password",password);
        testCred.putHiddenInput("HostAuth_password_confirm",password);
        testCred.putHiddenInput("HostAuth_authtype",authType);
        testCred.putHiddenInput("RestrictionType","limit");
        testCred.putHiddenInput("scopename", "DummyScopeSetID");
        testCred.putHiddenInput("ipAddress",ipadd);
        testCred.putHiddenInput("ClassName","com.collation.platform.security.auth.HostAuth");
        testCred.putHiddenInput("testCredential","true");
        if (elementFileID != null) 
        {
            testCred.setEnctypeAttribute( 'multipart/form-data' );
            sshFileElement.name = "HostAuth_sshKey";
            sshFileElement.id   = "HostAuth_sshKey";
            testCred.hiddenInputMap["HostAuth_sshKey"] = sshFileElement;
        }
        testCred.submit();
        $( testCred ).on( 'executionSucceeded', function( event )
        {
          
          restoreID();
          tsa.clearSysMessages();        
          tsa.printSysMessage( { level   : 'info'
                                  , message : tsa.bind( tsa.i18n.labelDiscoHMCCredsValid, [credName ,ipadd] ) } );
        } );
        
        $( testCred ).on( 'executionFailed', function( event )
        {
          restoreID();
          tsa.clearSysMessages();        
          tsa.printSysMessage( { level   : 'error'
                               , message : tsa.bind( tsa.i18n.labelDiscoHMCCredsInvalid, [credName, ipadd ] )} );
        } );
    };
    //  -------------------------------------------------------------------------
    return tsa.cmd.handlers.ScopesetshmcHandler;
  }

);
