/*
 * IBM Technical Support Appliance Version 2 Release 5 Modification 0
 *
 * (C) Copyright IBM Corp. 2019
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/IconFactory'
  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , TablePanel
          , ButtonFactory
          , FormPanel
          , ShowCommand
          , Command
          , IconFactory)
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'ScopeshmcHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.ScopeshmcHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.ScopeshmcHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ScopeshmcHandler.prototype ).constructor = tsa.cmd.handlers.ScopeshmcHandler;
    // Exclusion Form Panel
    tsa.cmd.handlers.ScopeshmcHandler.prototype.fpExclusion         = null;

    tsa.cmd.handlers.ScopeshmcHandler.prototype.noOfExclusionRows          = 0;

    //--------------------------------------------------------------------------------------
    tsa.cmd.handlers.ScopeshmcHandler.prototype.addExclusionRow = function(){
      $(this.fpExclusion.layoutTable).css('width', '');
      /** @type {Element} */ var row     = document.createElement( 'tr' );
      /** @type {Element} */ var cell    = document.createElement( 'td' );
      /** @type {Element} */ var cell2   = document.createElement( 'td' );
      /** @type {Element} */ var select  = document.createElement( 'select' );
      /** @type {Element} */ var aDel    = $("<a class ='deleteRow' href='#' style='text-decoration: none' ></a>").get(0);
      /** @type {Element} */ var option  = document.createElement( 'option' );
      $( option ).attr( 'selected', 'selected' );
      $( aDel ).on("click", function(e){
        tsa.clearSysMessages();
        $(this).closest('tr').remove();
        });
      /** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete' , caption :  'Delete Exclusion'} );
      $(aDel).append(deleteIcon);
      $( select ).attr("href","#");
      $( option ).val("excAddress");
      $( option ).text("Host");
      $( select ).append(option);

      option     = document.createElement( 'option' );
      $(option).val("excRange");
      $(option).text("Range");
      $( select ).append(option);

      option     = document.createElement( 'option' );
      $(option).val("excNetwork");
      $(option).text("Subnet");
      $( select ).append(option);

      $( cell ).append( aDel     );
      $( cell ).append( tsa.NBSP );
      $( cell ).append( select   );
      $( row  ).append(  cell    );


      this.buildExclusionHost(cell2,this.noOfExclusionRows,this.fpExclusion,null,null);

      /** type{function}*/
      var ipRule = tsa.cmd.handlers.ScopeshmcHandler.prototype.ipRule;

      this.fpExclusion.addValidationRule('excAddr['+this.noOfExclusionRows+'][ip]',ipRule);
      $(select).on("change", { noOfExclusionRows : this.noOfExclusionRows , exclusionForm : this.fpExclusion} , function(e){
        $(cell2).empty();
        /** @type {Element} */ var input  = null;
        if (this.value === 'excAddress'){

          tsa.cmd.handlers.ScopeshmcHandler.prototype.buildExclusionHost(cell2,e.data.noOfExclusionRows,e.data.exclusionForm,null,null);

        } else if (this.value === 'excRange'){
          tsa.cmd.handlers.ScopeshmcHandler.prototype.buildExclusionRange(cell2,e.data.noOfExclusionRows,e.data.exclusionForm,null,null);

        } else if(this.value === 'excNetwork'){
          tsa.cmd.handlers.ScopeshmcHandler.prototype.buildExclusionSubnet(cell2,e.data.noOfExclusionRows,e.data.exclusionForm,null,null);
        }
      });
      $(cell).append(aDel);
      $(cell).append(select);
      $(row).append( cell );
      $(row).append( cell2 );

      this.fpExclusion.layoutTableBody.appendChild( row );

      this.noOfExclusionRows+=1;

    };

    //--------------------------------------------------------------------------------------
    tsa.cmd.handlers.ScopeshmcHandler.prototype._edit = function(){
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( { title         : i18n.labelDiscoScopesHeading
                                                                                                 , helpDoc       : 'sshelp_discovery_hmc_scopes_edit_panel.html' } );
//      /** @type {tsa.widgets.FormPanel}   */     var fpDiscoveryOptions      = new FormPanel( { title  : i18n.labelDiscoScopesSelectHeading
//                                                                                              , action : 'scopes/ajaxEditScopeElement' } );
      /** @type {tsa.widgets.FormPanel}   */     var fpAdresses              = new FormPanel( { title  : i18n.labelDiscoScopesAddressHeading
                                                                                                , action : 'scopes/ajaxEditScopeElement'
                                                                                                , description: i18n.labelDiscoScopesAddressDesc } );
      /** @type {tsa.cmd.Command} */             var getScopeElementCmd      = new Command({  clearSystemMessages : false });
      /** @type {string}                  */     var pid                     = this.getParameter('pid');
      /** @type {string}                  */     var sepid                   = this.getParameter('sepid');
      /** @type {element}        */              var buttonSeparator         = null;
      this.fpExclusion                                                       = new FormPanel( { title  : i18n.labelDiscoScopeExclusionHeading
                                                                                                  , description: i18n.labelDiscoScopeExclusionDesc } );
      /** @type {String}        */              var ipAdress                 = null;
      
      /** @type {function()}        */
      var changeHandler = function(e){
        tsa.clearSysMessages();
        // Remove validations
        $(fpAdresses.layoutTableBody).empty();
        fpAdresses.removeValidation("aaddr");
        fpAdresses.removeValidation("rstart");
        fpAdresses.removeValidation("rend");
        fpAdresses.removeValidation("rdesc");
        fpAdresses.removeValidation("naddr");
        fpAdresses.removeValidation("nmask");
        fpAdresses.removeValidation("ndesc");

        if (this.value === 'address'){

          $(buttonSeparator).hide();
          /** @type {element} */ var ipAdressElement = $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0);
          $( ipAdressElement ).attr("value", ipAdress);
          fpAdresses.setTitle(i18n.labelDiscoScopesAddressHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesAddressDesc);
          fpAdresses.appendFieldRow( {  label    : i18n.labelDiscoScopesAddressAddress
                                      , required : true
                                      , element  :  ipAdressElement
                                      , hint     : i18n.labelDiscoScopesAddressAddressDesc } );
          e.data.fpExclusion.hide();
        }
      };
      //Do not show exclusion panel by default
      this.fpExclusion.hide();
      fpAdresses.putHiddenInput("scopeset",pid);
      fpAdresses.putHiddenInput("sepid",sepid);
      fpAdresses.putHiddenInput("option","address");

      fpAdresses.linkTo(this.fpExclusion);

      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer = ButtonFactory.buildButtonContainer();
      /** @type {element}                   */   var selectIP        = $("<select name='option' id='option' size='1' ></select>").get(0);
      /** @type {element}                   */   var option          = document.createElement( 'option' );
      $( option ).attr('value', 'address');
      $( option ).text(i18n.labelDiscoScopesSelectAddress);
      $( selectIP ).append(option);

      option   = document.createElement('option');
      $( option ).attr('value', 'range');
      $( option ).text(i18n.labelDiscoScopesSelectRange);
      $( selectIP ).append(option);

      option   = document.createElement('option');
      $( option ).attr('value', 'network');
      $( option ).text(i18n.labelDiscoScopesSelectNetwork);

      $(selectIP).append(option);


      $( selectIP ).on('change'   ,{ fpExclusion : this.fpExclusion }, changeHandler );
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonSave
            , onclick : $.proxy(function( event )
              {
              fpAdresses.putHiddenInput("noOfRows",this.noOfExclusionRows);
              fpAdresses.submit();
              $( fpAdresses ).on( 'executionSucceeded', function( event )
              {
                (new ShowCommand( {
                  action : 'scopesetshmc/show'
                    , clearSystemMessages : false
                    , parameters : { pid    : sepid
                      }
                })).execute();
              } );
              // Show errors
              $( fpAdresses ).on( 'executionFailed', function( event )
                  {
                    if( TV.typeIsObject( typeof event.response.payload.errors ) )
                    {
                      for( /** @type {string} */ var fieldName in event.response.payload.errors )
                      {
                        fpAdresses.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                      }
                      delete event.response.payload.errors;
                    }
                  } );


              },this)
            }
          )
        );
      getScopeElementCmd.setAction("scopes/getScopeElement/" + pid);
      getScopeElementCmd.execute();
      $(getScopeElementCmd).on("executionSucceeded", $.proxy(function(event){
        /** @type {Object} */ var scopeElement  = event.response.payload.scopeElement;
        if (scopeElement.type === 0){
          $( selectIP ).val("address");
          ipAdress = scopeElement.ipAddress;
          /** @type {element}*/ var inputAddress  = $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text' >").get(0);
          $( inputAddress ).attr("value", scopeElement.ipAddress);
          fpAdresses.setTitle(i18n.labelDiscoScopesAddressHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesAddressDesc);
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesAddressAddress
                                                  , required : true
                                                  , element  :  inputAddress
                                                  , hint     : i18n.labelDiscoScopesAddressAddressDesc } );

          $( buttonSeparator ).hide();
          this.fpExclusion.hide();
        }
      },this));
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );
      buttonSeparator=ButtonFactory.buildButtonSeparator ;

      $(buttonSeparator).hide();

      $( buttonContainer ).append(buttonSeparator)
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'cancel'
            , grayout : true
            , label   : i18n.buttonCancel
            , onclick : function( event )
              {
                (new ShowCommand( {
                    action : 'scopesetshmc/show'
                        , clearSystemMessages : false
                        , parameters : { pid    : sepid }
                	}
                 )).execute();
              }
            }
          )
        );

      $(fpAdresses.form).attr("aria-label", i18n.labelDiscoScopesAddressHeading);
      $(this.fpExclusion.form).attr("aria-label", i18n.labelDiscoScopeExclusionHeading);
      contentArea.append( fpAdresses.getDomNode() );
      contentArea.append( this.fpExclusion.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
    };
    tsa.cmd.handlers.ScopeshmcHandler.prototype.ipRule = function( value ){
      /** @type {object} */  var regEx     = /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/;
      /** @type {string} */  var matchedIp = "";
      /** @type {boolean} */ var valid     = true;
      if( value != "" ) {
        matchedIp = value.match(regEx);
        if(matchedIp != null) {
          /** @type {Array[string]} */ var split = matchedIp[0].split('.');
          for (/** @type {integer} */ var i = split.length - 1; i >= 0; i--) {
            /** @type {string} */ var s = split[i];
            if(s.length == 0 || isNaN(s) || s < 0 || s >255)
               valid = false;
         };

        } else {
          valid = false;
       }
      }
      return ( valid ? null : "Invalid IP" );
    };
    tsa.cmd.handlers.ScopeshmcHandler.prototype.buildExclusionHost = function( cell, noOfRows, exclusionForm , exclusion, defaultIP){

      exclusionForm.addValidationRule('excAddr['+noOfRows+'][ip]', tsa.cmd.handlers.ScopeshmcHandler.prototype.ipRule);
      /** @type {Element} */ var input = $("<input type='text' size='15' id='excAddr' placeholder='IP address' data-index='' class='ibm-text ipscope'>").get(0);
      if( exclusion !=null){
        $( input ).attr( 'value', exclusion.ipAddress);
      }else if(defaultIP!=null){
        $( input ).attr( 'value', defaultIP);
      }
      $( input ).attr( 'name', 'excAddr['+noOfRows+'][ip]' );
      $(cell).append(input);
      input   = $("<input type='hidden' name='dummy'>").get(0);
      $(cell).append(input);
      input   = $("<input  size='30' placeholder='Description' data-index='' class='ibm-text' type='text'>").get(0);
      $( input ).attr('name', 'excAddrDesc[' + noOfRows + '][ip]');
      if(exclusion!=null) {
        $( input ).attr( 'value', exclusion.name );
        }
      $(cell).append(input);
    };
    tsa.cmd.handlers.ScopeshmcHandler.prototype.buildExclusionRange = function( cell, noOfRows, exclusionForm , exclusion, defaultIP){

      /** type{function}*/
      var ipRule = tsa.cmd.handlers.ScopeshmcHandler.prototype.ipRule;

      exclusionForm.addValidationRule('excRstart['+noOfRows+'][range]', ipRule);

      input = $("<input size='15'  placeholder='Starting IP' aria-required='true' data-index='' class='ibm-text  ipscope' type='text'>").get(0);
      if( exclusion !=null ){
        $( input ).attr( 'value', exclusion.start );
      }else if (defaultIP!=null){
        $( input ).attr( 'value', defaultIP );
      }
      $( input ).attr('name', 'excRstart['+noOfRows+'][range]');

      $(cell).append(input);

      input   = $("<input size='15' placeholder='Ending IP' aria-required='true' data-index='' class='ibm-text  ipscope' type='text'>").get(0);

      $( input ).attr( 'name', 'excRend['+noOfRows+'][range]' );
      if( exclusion !=null ){
        $( input ).attr( 'value', exclusion.end );
      }
      exclusionForm.addValidationRule('excRend['+noOfRows+'][range]', ipRule);
      $(cell).append(input);

      input   = $("<input size='25' placeholder='Description' class='ibm-text' type='text'>  ").get(0);
      $( input ).attr( 'name', 'excRdesc['+noOfRows+'][range]' );
      if( exclusion !=null ){
        $( input ).attr( 'value', exclusion.name );
      }
      $(cell).append(input);
    };
    tsa.cmd.handlers.ScopeshmcHandler.prototype.buildExclusionSubnet = function(  cell, noOfRows, exclusionForm, exclusion, defaultIP){
      /** type{function}*/
      var ipRule = tsa.cmd.handlers.ScopeshmcHandler.prototype.ipRule;
      input = $("<input size='15' placeholder='Subnet' aria-required='true' data-index='' class='ibm-text  ipscope' type='text'>").get(0);
      $( input ).attr( 'name', 'excNaddr['+noOfRows+'][subnet]' );

      if(exclusion!=null){
        $( input ).attr( 'value', exclusion.ipAddress );
      }else if(defaultIP!=null){
        $( input ).attr( 'value', defaultIP );
      }
      exclusionForm.addValidationRule('excNaddr['+noOfRows+'][subnet]', ipRule);
      $(cell).append(input);

      input   = $("<input size='15' placeholder='Subnet mask' aria-required='true' data-index='' class='ibm-text  ipscope' type='text'>").get(0);
      $( input ).attr( 'name', 'excNmask['+noOfRows+'][subnet]' );
      if(exclusion!=null){
        $( input ).attr( 'value', exclusion.subnetMask );
      }
      exclusionForm.addValidationRule('excNmask['+noOfRows+'][subnet]', ipRule);
      $(cell).append(input);

      input   = $("<input size='25' placeholder='Description' class='ibm-text' type='text'> ").get(0);
      $( input ).attr( 'name', 'excDesc['+noOfRows+'][subnet]' );
      if(exclusion!=null){
        $( input ).attr( 'value', exclusion.name );
        }
      $(cell).append(input);
    };
    return tsa.cmd.handlers.ScopeshmcHandler;
  }


);
