/*
 * IBM Technical Support Appliance Version 2 Release 5 Modification 0
 *
 * (C) Copyright IBM Corp. 2019
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , IconFactory
          , TablePanel
          , ButtonFactory
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'UpdateHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.UpdateHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.UpdateHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.UpdateHandler.prototype ).constructor = tsa.cmd.handlers.UpdateHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.UpdateHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        			= tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea 			= null;
      /** @type {tsa.widgets.FormPanel}   */ var checkUpdateFormPanel 	= null;
      /** @type {tsa.widgets.FormPanel}   */ var docFormPanel  			= null;
      /** @type {Element}                 */ var checkUpdateButton    	= null;
      /** @type {Element}                 */ var docButton    			= null;
      
      
      // Initialize the content area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelUpdateHeading
                                      , helpDoc : 'sshelp_update_panel.html' } );
      // Append description the content area...
      contentArea.appendMessage( { text : i18n.labelUpdateCaption } );
      
      
      //UPDATE FORM
      checkUpdateFormPanel = new FormPanel( { title  : i18n.labelUpdateCheck4Heading } );
      checkUpdateFormPanel.setDescription( i18n.labelUpdateCheck4Desc );
      checkUpdateButton = ButtonFactory.buildButton ({
            type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonUpdateCheck4Update
          , onclick : function( event ){
        	  ( new ShowCommand( { action : 'update/check' } ) ).execute();
          }
      });

      var updateDiv = document.createElement( 'div' );
      $( updateDiv ).append( checkUpdateButton );
      checkUpdateFormPanel.appendJointRow( updateDiv );
      
      //DOCUMENTATION FORM
      docFormPanel = new FormPanel( { title  : i18n.labelUpdateGetDocHeading } );
      docFormPanel.setDescription( i18n.labelUpdateGetDocDesc );
      docButton = ButtonFactory.buildButton ({
            type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonUpdateGetDoc
          , onclick : function( event ){
        	  ( new ShowCommand( { action : 'documentation/index' } ) ).execute();
            }
      });

      var docDiv = document.createElement( 'div' );
      $( docDiv ).append( docButton );
      docFormPanel.appendJointRow( docDiv );
      
      
      //Link forms
      checkUpdateFormPanel.linkTo( docFormPanel );
      
      //Append forms
      contentArea.append( checkUpdateFormPanel.getDomNode() );
      
      this.show( contentArea.getDomNode() );
    };
    
    
    //-------------------------------------------------------------------------
    /**
     * Show update schedule screen.
     */
    tsa.cmd.handlers.UpdateHandler.prototype._check = function()
    {
      /** @type {Object}                    */ var i18n             = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea      = null;
      /** @type {tsa.widgets.FormPanel}     */ var updateFormPanel 	= null;
      /** @type {tsa.widgets.TablePanel}    */ var updateTablePanel = null; 
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer  = ButtonFactory.buildButtonContainer();
      /** @type {string}                    */ var fromLicense      = this.getParameter ( 'fromlicense' );
      /** @type {tsa.cmd.Command}           */ var checkUpdateCmd   = new Command( { action              : 'update/getUpdate'
                                                                                   , clearSystemMessages : false } );
    

      // Initialize the content area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelUpdateCheckHeading
                                      , helpDoc : 'sshelp_update_panel.html' } );
      // Append description the content area...
      contentArea.appendMessage( { text : i18n.labelUpdateCheckDesc } );
      
      //check update status
      $( checkUpdateCmd ).on( 'executionSucceeded', function( event ){
    	  var updateInfo  = event.response.payload.updates;
          
    	  
    	  if(updateInfo == null){ //no update exists
    		  updateFormPanel = new FormPanel( { title : i18n.labelUpdateCheckAvailHeading } );
    		  
    		  var src  = tsa.attributes.imagesPath + '/' + tsa.i18n[ 'imageStatusInfo16' ];
    		  var icon = document.createElement( 'img' );
     		  var div  = document.createElement( 'div' );
    		  var span = document.createElement( 'span' );
    		  
              $( icon ).attr( 'alt', src );
              $( icon ).attr( 'src', src  );
              $( icon ).css( { 'vertical-align' : 'middle' } );
              $( span ).css( { 'white-space' : 'nowrap', 'font-size' : 'small', 'font-style' : 'italic' } );
              $( span ).append( icon );
     		  $( span ).append( document.createTextNode( tsa.NBSP + i18n.labelUpdateCheckNoUpdatesAvailable ) );
     		  $( div  ).append( span );
    		  
    		  updateFormPanel.appendJointRow( div );
    		  contentArea.append( updateFormPanel.getDomNode() );
		  
    	  }else{ //an update exists
    		  
    	      var name 		    = i18n.labelUpdateGenericDescription;
    		  var releaseDate   = updateInfo.releaseDate;
    		  var version       = updateInfo.version;
    		  var readmeDetails = updateInfo.readmeDetails;
    		  var updateLicense = event.response.payload.licenseRequired;
    		  updateFormPanel = new FormPanel( { border : false } );
    		  
    		  contentArea.appendMessage( { level : 'INFO'  , border : true , text : i18n.labelUpdateCheckRestartsAfter } );
    		  updateTablePanel = new TablePanel( { title       : i18n.labelUpdateCheckAvailHeading
    			  								 , description : i18n.labelUpdateCheckUpdateAvailable
    			  								 , columns     : [ { id : 'property' , label : '' , width : '20%' , sortable : false } ] } );
    		 
    		  var nameSpan = document.createElement( 'span' );
    		  $( nameSpan ).css( { 'font-weight' : 'bold' } );
    		  $( nameSpan ).append( name );
    		  
    		  updateTablePanel.appendRow( [ i18n.labelUpdateCheckUpdateDesc+":", nameSpan ] );
    		  updateTablePanel.appendRow( [ i18n.labelUpdateCheckUpdateDate+":", releaseDate ] );
    		  updateTablePanel.appendRow( [ i18n.labelUpdateCheckUpdateVersion+":", version ] );
    		  contentArea.append( updateTablePanel.getDomNode() );
    		 
    		  
    		  //details overlay
    		  var div1   = document.createElement( 'div' );
    		  var div2   = document.createElement( 'div' );
    	      var p      = document.createElement( 'p'   );
    	      var aClose = $( '<a class="ibm-common-overlay-close" href="#close" style="text-decoration: none">'+i18n.buttonOverlayClose + '</a>').get(0);
    	      
    	      $( aClose ).on("click", function(e){ $( div1 ).hide(); });
    	         
    	      $( div1 ).addClass( 'ibm-common-overlay  ibm-overlay-alt-two' );
    	      $( div1 ).attr( 'id', 'overlay750' );
    	      $( div1 ).css( { 'font-color' : '#666666' } );
    	      $( div2 ).addClass( 'ibm-head');
    	      $( p    ).append( aClose );
    	      $( div2 ).append( p );
    	      $( div1 ).append( div2 );
    	          
    	      var div3  = document.createElement( 'div' );
    	      $( div3 ).addClass( 'ibm-body' );
    	      	
    	      var div4 = $('<div class="ibm-main ibm-code-sample" style="height:30em; background-color:#ffffff !important; border:0px; margin:0px; overflow:scroll" ></div>').get(0);
    	     
    	      var a2    = $( '<a class="ibm-access"><!-- Accessibility anchor --></a>' ).get(0);
    	      $( div4 ).append( a2 );
    	        
    	      var div5  = document.createElement( 'div' );
    	      $( div5 ).attr( 'id', 'updateDetailsTitle' );
    	        
    	      var h1  = document.createElement( 'h1' );
    	      $( h1 ).append(i18n.labelUpdateViewDetailsTitle)
    	      $( div5 ).append( h1 )
    	      $( div4 ).append( div5 );
    	        
    	      var div6  = document.createElement( 'div' );
    	      $( div6 ).addClass( 'ibm-container ibm-alternate' );
    	        
    	      var div7  = document.createElement( 'div' );
    	      $( div7 ).addClass( 'ibm-container-body' );
    	      $( div7 ).attr( 'id', 'updateDetailsList' );
    	      
    	      var pre = $('<pre>' + i18n.labelUpdateEnhancementDesc + '</pre>').get(0);
    	      $( div7 ).append( pre );
    	        
    	      var ul = document.createElement( 'ul' );
    	      
    	      //update details & history
    	      if( readmeDetails !== null )
    	      {
    	    	  var detail = null;
    	          $.each( readmeDetails, function( entryIndex, entry )
    	          {
    	        	  var p2 = document.createElement( 'p' );
    	        	  
    	        	  detail = $( '<li style="font-weight: bold; font-size:small"></li>' ).get(0);
    	        	  $( detail ).text( entry.heading );
    	        	  $( p2 ).append( detail );
    	        	  
    	        	  var p3 = document.createElement( 'p' );
    	        	  $( p3 ).attr( 'id', 'update-date' );
    	        	  $( p3 ).append( entry.date );
    	        	  $( p2 ).append( p3 );
    	        	  
    	        	  var ul2 = document.createElement( 'ul' );
    	        	  $.each( entry.enhancements, function( enhancementIndex, enhancement )
    	        	  {
    	        		  var enhancementText = $( '<li></li>' ).get(0);
    	        		  $( enhancementText ).css( { 'font-size' : '140%', 'font-color' : '#666666' } )
    	        		  $( enhancementText ).text( enhancement );
    	        		  $( ul2 ).append( enhancementText );
    	        		  $( p2 ).append( ul2 ); 
    	        	  });
    	        	  
    	        	 $( ul ).append( p2 );
    	        	  
    	          });
    	        
    	          $( div7 ).append( ul );
    	          $( div6 ).append( div7 );
    	          $( div4 ).append( div6 );
    	          $( div3 ).append( div4 );
    	          $( div1 ).append( div3 );
    	         
    	          var div8  = document.createElement( 'div' );
    	          $( div8 ).addClass( 'ibm-footer' );
    	          $( div1 ).append( div8 );
    	          
    	        }
    	       
    	      $( div1 ).hide();
    	      
    	      //append details overlay  
    	      updateFormPanel.appendJointRow( div1 );
    	        
    	      //View and Accept License button
    	      var viewLicenseCmd = new Command({ action : 'update/ajaxViewLicense', clearSystemMessages : false,  blockUI : false });
      		  var updateLicenseButton = ButtonFactory.buildButton({ 
      			    type    : 'arrow'  
      		      , label   : i18n.buttonViewLicenseNow 
      		      , onclick : function( event ){
        	  				 viewLicenseCmd.execute();
        	      }
      	      });
      		  $( viewLicenseCmd ).on( 'executionSucceeded', function( event ){
  	  			  var updateLicenseVRM = event.response.payload.updateLicenseVRM;
  	  			  ( new ShowCommand( { action : 'updatelicense/index' , clearSystemMessages : false 
  	  			                     , parameters : { "updateLicenseVRM" : updateLicenseVRM } } ) ).execute();
  	  		  });

    	      //perform update button
    	      var updateCmd = new Command({ action : 'update/ajaxPerform', clearSystemMessages : false,  blockUI : false });
      		  var updateButton = ButtonFactory.buildButton({ 
      			    type    : 'arrow'  
      		      , label   : i18n.buttonUpdateCheckUpdateNow 
      		      , onclick : function( event ){
      		    	  		 updateCmd.execute();
      		      }
      	      });
      	      
      		  $( updateCmd ).on( 'executionSucceeded', function( event ){
  	  			  ( new ShowCommand( { action : 'main/index' , clearSystemMessages : false  } ) ).execute();
  	  		  });
      		  
      		  //view documentation button
      		  var viewButton = ButtonFactory.buildButton({ 
      			  	type    : 'arrow' 
      			  , label   : i18n.buttonUpdateViewDetails 
      			  , onclick : function( event ){
      					  ibmCommon.Overlays.show('overlay750', this);
      			  }
      		  });
			  if ( updateLicense ){
	      	      	updateTablePanel.setDescription( i18n.labelUpdateCheckLicenseAvailable );
	      	      	$( updateLicenseButton ).show();
	      	      	$( updateButton ).hide();
  			  	}else{
  			  		updateTablePanel.setDescription( i18n.labelUpdateCheckUpdateAvailable );
  			  		$( updateButton ).show();
  			  		$( updateLicenseButton ).hide();
  			  	}

			  $( buttonContainer ).append( updateLicenseButton );
      	      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      	      $( buttonContainer ).append( updateButton );
      	      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      	      $( buttonContainer ).append( viewButton );
      	      
      	      updateFormPanel.appendJointRow( buttonContainer );
      	      contentArea.append( updateFormPanel.getDomNode() );
    	     
    	  }
      });
      
      checkUpdateCmd.setParameter("fromlicense", fromLicense);
      
      // check for update
      checkUpdateCmd.execute();
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.UpdateHandler;
  }
);