/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  ],
  
  function( $
          , TV
          )
  {
    //---------------------------------------------------------------------------
    
    /**
     * This module is not a constructor, but a set of data and functions.
     */
    tsa.data.TimeZones = {};
    
    //---------------------------------------------------------------------------
    
    /**
     * The data, as an array, sorted by GMT offset.
     * @private
     * @type {Array.<Object>}
     */
    tsa.data.TimeZones._data =
    [    { id : 'GB'                   , name : { en : '+0:00 - Greenwich Mean Time' } }
       , { id : 'CET'                  , name : { en : '+1:00 - Central European Time' } }
       , { id : 'EET'                  , name : { en : '+2:00 - Eastern European Time (Europe)' } }
       , { id : 'Asia/Amman'           , name : { en : '+2:00 - Eastern European Time (Asia)' } }
       , { id : 'Asia/Jerusalem'       , name : { en : '+2:00 - Israel Time' } }
       , { id : 'Africa/Nairobi'       , name : { en : '+3:00 - Eastern African Time' } }
       , { id : 'Iran'                 , name : { en : '+3:30 - Iran Time' } }
       , { id : 'Asia/Baku'            , name : { en : '+4:00 - Russia Zone 3' } }
       , { id : 'Asia/Kabul'           , name : { en : '+4:30 - Afghanistan Time' } }
       , { id : 'Etc/GMT-5'            , name : { en : '+5:00 - Russia Zone 4' } }
       , { id : 'Asia/Calcutta'        , name : { en : '+5:30 - India Time' } }
       , { id : 'Asia/Kathmandu'       , name : { en : '+5:45 - Nepal Time' } }
       , { id : 'Etc/GMT-6'            , name : { en : '+6:00 - Bangladesh' } }
       , { id : 'Asia/Rangoon'         , name : { en : '+6:30 - Cocos Islands Time' } }
       , { id : 'Asia/Jakarta'         , name : { en : '+7:00 - West Indonesian Time' } }
       , { id : 'Asia/Taipei'          , name : { en : '+8:00 - China Time' } }
       , { id : 'Australia/Perth'      , name : { en : '+8:00 - Australian Western Time' } }
       , { id : 'Japan'                , name : { en : '+9:00 - Japan Time' } }
       , { id : 'Australia/Adelaide'   , name : { en : '+9:30 - Australian Central Time' } }
       , { id : 'Australia/Sydney'     , name : { en : '+10:00 - Australian Eastern Time' } }
       , { id : 'Etc/GMT-11'           , name : { en : '+11:00 - Soloman Islands' } }
       , { id : 'Pacific/Auckland'     , name : { en : '+12:00 - International Date Line East' } }
       , { id : 'Pacific/Fiji'         , name : { en : '+12:00 - Fiji Time' } }
       , { id : 'Pacific/Chatham'      , name : { en : '+12:45 - Chatham Island Time' } }
       , { id : 'Pacific/Tongatapu'    , name : { en : '+13:00 - Nukualofa' } }
       , { id : 'Etc/GMT+12'           , name : { en : '-12:00 - International Date Line West' } }
       , { id : 'US/Samoa'             , name : { en : '-11:00 - Samoa Time' } }
       , { id : 'HST'                  , name : { en : '-10:00 - Hawaii Time' } }
       , { id : 'US/Alaska'            , name : { en : '-9:00 - Alaska Time' } }
       , { id : 'PST8PDT'              , name : { en : '-8:00 - Pacific Time' } }
       , { id : 'MST7MDT'              , name : { en : '-7:00 - Mountain Time (US and Canada)' } }
       , { id : 'Mexico/BajaSur'       , name : { en : '-7:00 - Mountain Time (Mexico)' } }
       , { id : 'CST6CDT'              , name : { en : '-6:00 - Central Time (US and Canada)' } }
       , { id : 'Mexico/General'       , name : { en : '-6:00 - Central Time (Mexico)' } }
       , { id : 'EST5EDT'              , name : { en : '-5:00 - Eastern Time' } }
       , { id : 'America/Havana'       , name : { en : '-5:00 - Cuba Time' } }
       , { id : 'Canada/Atlantic'      , name : { en : '-4:00 - Atlantic Time' } }
       , { id : 'Chile/Continental'    , name : { en : '-4:00 - Chile Time' } }
       , { id : 'Brazil/West'          , name : { en : '-4:00 - Western Brazil' } }
       , { id : 'America/Asuncion'     , name : { en : '-4:00 - Paraguay Time' } }
       , { id : 'Canada/Newfoundland'  , name : { en : '-3:30 - Newfoundland Time' } }
       , { id : 'America/Buenos_Aires' , name : { en : '-3:00 - Argentine Time' } }
       , { id : 'Brazil/East'          , name : { en : '-3:00 - Eastern Brazil' } }
       , { id : 'America/Montevideo'   , name : { en : '-3:00 - Uruguay Time' } }
       , { id : 'Etc/GMT+2'            , name : { en : '-2:00 - Mid-Atlantic' } }
       , { id : 'Atlantic/Azores'      , name : { en : '-1:00 - Azores Time' } }
    ];
    
    //---------------------------------------------------------------------------
    
    /**
     * The data, as a map (by ID).
     * @private
     * @type {Object.<string,Object>}
     */
    tsa.data.TimeZones._dataMap = {};
    
    $.each( tsa.data.TimeZones._data, function( index, timezone )
    {
      tsa.data.TimeZones._dataMap[ timezone.id ] = timezone;
    } );
    
    //---------------------------------------------------------------------------
    
    /**
     * Access method: get TimeZone by ID.
     * @param {string} id The TimeZone ID.
     * @return {Object}   The TimeZone data, or null if ID is not found.
     */
    tsa.data.TimeZones.getById = function( id )
    {
      /** @type {Object} */var timezone = null;
      
      if( TV.typeIsString( typeof id ) )
      {
        if( TV.typeIsObject( typeof tsa.data.TimeZones._dataMap[ id ] ) )
        {
          timezone = tsa.data.TimeZones._dataMap[ id ];
        }
      }
      
      return timezone;
    };
    
    //---------------------------------------------------------------------------
    
    /**
     * Access method: get TimeZones sorted by GMT offset.
     * @return {Array.<Object>} The TimeZones data.
     */
    tsa.data.TimeZones.getTimeZones = function()
    {
      return tsa.data.TimeZones._data.slice();
    };
    
    //---------------------------------------------------------------------------
    
    return tsa.data.TimeZones;
  }
);